@@ -1,4 +1,4 @@
-computeSigmaHat <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE, 
+computeSigmaHat <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
                             delta = TRUE, debug = FALSE) {
 
     # state or final?
@@ -19,7 +19,7 @@ computeSigmaHat <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
         MLIST <- GLIST[mm.in.group]
 
         if(representation == "LISREL") {
-            Sigma.hat[[g]] <- computeSigmaHat.LISREL(MLIST = MLIST, 
+            Sigma.hat[[g]] <- computeSigmaHat.LISREL(MLIST = MLIST,
                                                      delta = delta)
         } else {
             stop("only representation LISREL has been implemented for now")
@@ -54,14 +54,13 @@ computeSigmaHat <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
 ## only if conditional.x = TRUE
 ## compute the (larger) unconditional 'joint' covariance matrix (y,x)
 ##
-## Sigma (Joint ) = [ (S11, S12), 
+## Sigma (Joint ) = [ (S11, S12),
 ##                    (S21, S22) ] where
-##     S11 = Sigma.res + PI %*% cov.x %*% t(PI) 
+##     S11 = Sigma.res + PI %*% cov.x %*% t(PI)
 ##     S12 = PI %*% cov.x
 ##     S21 = cov.x %*% t(PI)
 ##     S22 = cov.x
 computeSigmaHatJoint <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
-                                 lavsamplestats = NULL,
                                  delta = TRUE, debug = FALSE) {
 
     stopifnot(lavmodel@conditional.x)
@@ -87,7 +86,7 @@ computeSigmaHatJoint <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
             res.Sigma  <- computeSigmaHat.LISREL(MLIST = MLIST, delta = delta)
             res.int    <- computeMuHat.LISREL(MLIST = MLIST)
             res.slopes <- computePI.LISREL(MLIST = MLIST)
-            S.xx       <- lavsamplestats@cov.x[[g]]
+            S.xx       <- MLIST$cov.x
 
             S.yy <- res.Sigma + res.slopes %*% S.xx %*% t(res.slopes)
             S.yx <- res.slopes %*% S.xx
@@ -161,8 +160,7 @@ computeMuHat <- function(lavmodel = NULL, GLIST = NULL) {
 ## Mu (Joint ) = [ Mu.y, Mu.x ] where
 ##     Mu.y = res.int + PI %*% M.x
 ##     Mu.x = M.x
-computeMuHatJoint <- function(lavmodel = NULL, GLIST = NULL,
-                              lavsamplestats = NULL) {
+computeMuHatJoint <- function(lavmodel = NULL, GLIST = NULL) {
 
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
@@ -186,7 +184,7 @@ computeMuHatJoint <- function(lavmodel = NULL, GLIST = NULL,
             MLIST <- GLIST[ mm.in.group ]
             res.int    <- computeMuHat.LISREL(MLIST = MLIST)
             res.slopes <- computePI.LISREL(MLIST = MLIST)
-            M.x        <- lavsamplestats@mean.x[[g]]
+            M.x        <- MLIST$mean.x
 
             Mu.y <- res.int + res.slopes %*% M.x
             Mu.x <- M.x
@@ -221,7 +219,7 @@ computeTH <- function(lavmodel = NULL, GLIST = NULL) {
             TH[[g]] <- numeric(0L)
             next
         }
-   
+
         # which mm belong to group g?
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
 
@@ -318,8 +316,7 @@ computeGW <- function(lavmodel = NULL, GLIST=NULL) {
 # *unconditional* variance/covariance matrix of Y
 #  - same as Sigma.hat if all Y are continuous)
 #  - if also Gamma, cov.x is used (only if categorical)
-computeVY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
-                      diagonal.only = FALSE) {
+computeVY <- function(lavmodel = NULL, GLIST = NULL, diagonal.only = FALSE) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -336,17 +333,8 @@ computeVY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
 
-        if(!is.null(lavsamplestats)) {
-            cov.x <- lavsamplestats@cov.x[[g]]
-        } else {
-            if(lavmodel@fixed.x) {
-                stop("lavaaan ERROR: fixed.x = TRUE, but cov.x is NULL")
-            }
-            cov.x <- NULL
-        }
-
         if(representation == "LISREL") {
-            VY.g <- computeVY.LISREL(MLIST = MLIST, cov.x = cov.x)
+            VY.g <- computeVY.LISREL(MLIST = MLIST)
         } else {
             stop("only representation LISREL has been implemented for now")
         }
@@ -362,8 +350,8 @@ computeVY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
 }
 
 # V(ETA): latent variances variances/covariances
-computeVETA <- function(lavmodel = NULL, GLIST = NULL, 
-                        remove.dummy.lv = FALSE, lavsamplestats = NULL) {
+computeVETA <- function(lavmodel = NULL, GLIST = NULL,
+                        remove.dummy.lv = FALSE) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -380,17 +368,8 @@ computeVETA <- function(lavmodel = NULL, GLIST = NULL,
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
 
-        if(!is.null(lavsamplestats)) {
-            cov.x <- lavsamplestats@cov.x[[g]]
-        } else {
-            if(lavmodel@fixed.x) {
-                stop("lavaaan ERROR: fixed.x = TRUE, but cov.x is NULL")
-            }
-            cov.x <- NULL
-        }
-       
         if(representation == "LISREL") {
-            ETA.g <- computeVETA.LISREL(MLIST = MLIST, cov.x = cov.x)
+            ETA.g <- computeVETA.LISREL(MLIST = MLIST)
 
             if(remove.dummy.lv) {
                 # remove all dummy latent variables
@@ -432,7 +411,7 @@ computeVETAx <- function(lavmodel = NULL, GLIST = NULL) {
         if(representation == "LISREL") {
             lv.idx <- c(lavmodel@ov.y.dummy.lv.idx[[g]],
                         lavmodel@ov.x.dummy.lv.idx[[g]])
-            ETA.g <- computeVETAx.LISREL(MLIST = MLIST, 
+            ETA.g <- computeVETAx.LISREL(MLIST = MLIST,
                                          lv.dummy.idx = lv.idx)
         } else {
             stop("only representation LISREL has been implemented for now")
@@ -445,8 +424,8 @@ computeVETAx <- function(lavmodel = NULL, GLIST = NULL) {
 }
 
 # COV: observed+latent variances variances/covariances
-computeCOV <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
-    remove.dummy.lv = FALSE) {
+computeCOV <- function(lavmodel = NULL, GLIST = NULL,
+                       remove.dummy.lv = FALSE) {
 
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
@@ -464,10 +443,8 @@ computeCOV <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
         mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
         MLIST <- GLIST[ mm.in.group ]
 
-        cov.x <- lavsamplestats@cov.x[[g]]
-
         if(representation == "LISREL") {
-            COV.g <- computeCOV.LISREL(MLIST = MLIST, cov.x = cov.x)
+            COV.g <- computeCOV.LISREL(MLIST = MLIST)
 
             if(remove.dummy.lv) {
                 # remove all dummy latent variables
@@ -475,7 +452,7 @@ computeCOV <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
                             lavmodel@ov.x.dummy.lv.idx[[g]])
                 if(!is.null(lv.idx)) {
                     # offset for ov
-                    lambda.names <- 
+                    lambda.names <-
                         lavmodel@dimNames[[which(names(GLIST) == "lambda")[g]]][[1L]]
                     lv.idx <- lv.idx + length(lambda.names)
                     COV.g <- COV.g[-lv.idx, -lv.idx, drop=FALSE]
@@ -493,9 +470,9 @@ computeCOV <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
 
 
 # E(ETA): expectation (means) of latent variables (return vector)
-computeEETA <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL, 
+computeEETA <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
                         remove.dummy.lv = FALSE) {
-  
+
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -513,7 +490,7 @@ computeEETA <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
         MLIST <- GLIST[ mm.in.group ]
 
         if(representation == "LISREL") {
-            EETA.g <- computeEETA.LISREL(MLIST, 
+            EETA.g <- computeEETA.LISREL(MLIST,
                 mean.x=lavsamplestats@mean.x[[g]],
                 sample.mean=lavsamplestats@mean[[g]],
                 ov.y.dummy.lv.idx=lavmodel@ov.y.dummy.lv.idx[[g]],
@@ -540,9 +517,9 @@ computeEETA <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
 
 # E(ETA|x_i): conditional expectation (means) of latent variables
 # for a given value of x_i (instead of E(x_i))
-computeEETAx <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL, 
-                         eXo = NULL, remove.dummy.lv = FALSE) {
-  
+computeEETAx <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
+                         eXo = NULL, nobs = NULL, remove.dummy.lv = FALSE) {
+
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -562,12 +539,12 @@ computeEETAx <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
         EXO <- eXo[[g]]
         if(is.null(EXO)) {
             # create empty matrix
-            EXO <- matrix(0, lavsamplestats@nobs[[g]], 0L)
+            EXO <- matrix(0, nobs[[g]], 0L)
         }
 
         if(representation == "LISREL") {
-            EETAx.g <- computeEETAx.LISREL(MLIST, 
-                eXo=EXO, N=lavsamplestats@nobs[[g]],
+            EETAx.g <- computeEETAx.LISREL(MLIST,
+                eXo=EXO, N=nobs[[g]],
                 sample.mean=lavsamplestats@mean[[g]],
                 ov.y.dummy.lv.idx=lavmodel@ov.y.dummy.lv.idx[[g]],
                 ov.x.dummy.lv.idx=lavmodel@ov.x.dummy.lv.idx[[g]],
@@ -593,9 +570,9 @@ computeEETAx <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
 }
 
 # return 'regular' LAMBDA
-computeLAMBDA <- function(lavmodel = NULL, GLIST = NULL, 
+computeLAMBDA <- function(lavmodel = NULL, GLIST = NULL,
                           remove.dummy.lv = FALSE) {
-  
+
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -666,7 +643,7 @@ computeTHETA <- function(lavmodel = NULL, GLIST = NULL) {
 # E(Y): expectation (mean) of observed variables
 # returns vector 1 x nvar
 computeEY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL) {
-  
+
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -684,7 +661,7 @@ computeEY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL) {
         MLIST <- GLIST[ mm.in.group ]
 
         if(representation == "LISREL") {
-            EY.g <- computeEY.LISREL(MLIST = MLIST, 
+            EY.g <- computeEY.LISREL(MLIST = MLIST,
                                      mean.x=lavsamplestats@mean.x[[g]],
                                      sample.mean=lavsamplestats@mean[[g]],
                              ov.y.dummy.ov.idx=lavmodel@ov.y.dummy.ov.idx[[g]],
@@ -704,9 +681,9 @@ computeEY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL) {
 
 # E(Y | ETA, x_i): conditional expectation (means) of observed variables
 # for a given value of x_i AND eta_i
-computeYHAT <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL, 
-                        eXo = NULL, ETA = NULL, duplicate = FALSE) {
-  
+computeYHAT <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
+                        eXo = NULL, nobs = NULL, ETA = NULL, duplicate = FALSE) {
+
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -724,7 +701,7 @@ computeYHAT <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL,
         MLIST <- GLIST[ mm.in.group ]
 
         if(is.null(eXo[[g]]) && duplicate) {
-            Nobs <- lavsamplestats@nobs[[g]]
+            Nobs <- nobs[[g]]
         } else {
             Nobs <- 1L
         }
