@@ -1,5 +1,7 @@
 # model estimation
 lav_model_estimate <- function(lavmodel       = NULL,
+                               lavpartable    = NULL, # for parscale = "stand"
+                               lavh1          = NULL, # for multilevel + parsc
                                lavsamplestats = NULL,
                                lavdata        = NULL,
                                lavoptions     = NULL,
@@ -24,36 +26,188 @@ lav_model_estimate <- function(lavmodel       = NULL,
         PENV$PARTRACE <- matrix(NA, nrow=0, ncol=lavmodel@nx.free + 1L)
     }
 
-    # function to be minimized
-    minimize.this.function <- function(x, verbose=FALSE, infToMax=FALSE) {
-      
-        #cat("DEBUG: x = ", x, "\n")
+    # starting values (ignoring equality constraints)
+    x.unpack <- lav_model_get_parameters(lavmodel)
 
-        # current strategy: forcePD is by default FALSE, except
-        # if missing patterns are used
-        #if(any(lavsamplestats@missing.flag)) {
-        #    forcePD <- TRUE
-        #} else {
-            forcePD <- FALSE
+    # 1. parameter scaling (to handle data scaling, not parameter scaling)
+    parscale <- rep(1.0, length(x.unpack))
+
+    # for < 0.6 compatibility
+    if(is.null(lavoptions$optim.parscale)) {
+        lavoptions$optim.parscale <- "none"
+    }
+
+    if(lavoptions$optim.parscale == "none") {
+        # do nothing, but still set SCALE, as before
+    } else if(lavoptions$optim.parscale %in% c("stand", "st", "standardize",
+                                               "standarized", "stand.all")) {
+        # rescale parameters as if the data was standardized
+        # new in 0.6-2
+        #
+        # FIXME: this works well, as long as the variances of the
+        #        latent variables (which we do not know) are more or less
+        #        equal to 1.0 (eg std.lv = TRUE)
+        #
+        #        Once we have better estimates of those variances, we could
+        #        use them to set the scale
+        #
+
+        if(lavdata@nlevels > 1L) {
+            if(length(lavh1) > 0L) {
+                OV.VAR <- lapply(lavh1$implied$cov, diag)
+            } else {
+                OV.VAR <- lapply(do.call(c, lapply(lavdata@Lp, "[[", "ov.idx")),
+                                     function(x) rep(1, length(x) ))
+            }
+        } else {
+            if(lavoptions$conditional.x) {
+                OV.VAR <- lavsamplestats@res.var
+            } else {
+                OV.VAR <- lavsamplestats@var
+            }
+        }
+
+        if(lavoptions$std.lv) {
+            parscale <- lav_standardize_all(lavobject = NULL,
+                            est = rep(1, length(lavpartable$lhs)),
+                            est.std = rep(1, length(lavpartable$lhs)),
+                            cov.std = FALSE, ov.var = OV.VAR,
+                            lavmodel = lavmodel, lavpartable = lavpartable,
+                            cov.x = lavsamplestats@cov.x)
+        } else {
+            # needs good estimates for lv variances!
+            parscale <- lav_standardize_all(lavobject = NULL,
+                            est = rep(1, length(lavpartable$lhs)),
+                            #est.std = rep(1, length(lavpartable$lhs)),
+                            # here, we use whatever the starting values are
+                            # for the latent variances...
+                            cov.std = FALSE, ov.var = OV.VAR,
+                            lavmodel = lavmodel, lavpartable = lavpartable,
+                            cov.x = lavsamplestats@cov.x)
+        }
+
+        # in addition, take sqrt for variance parameters
+        var.idx <- which(lavpartable$op == "~~" &
+                         lavpartable$lhs == lavpartable$rhs)
+        if(length(var.idx) > 0L) {
+            parscale[var.idx] <- sqrt(abs(parscale[var.idx]))
+        }
+        parscale <- parscale[ lavpartable$free > 0 ]
+    }
+    # parscale should obey the equality constraints
+    if(lavmodel@eq.constraints && lavoptions$optim.parscale != "none") {
+        # pack
+        p.pack <- as.numeric( (parscale - lavmodel@eq.constraints.k0) %*%
+                               lavmodel@eq.constraints.K )
+        # unpack
+        parscale <- as.numeric(lavmodel@eq.constraints.K %*% p.pack) +
+                               lavmodel@eq.constraints.k0
+    }
+    if(debug) {
+        cat("parscale = ", parscale, "\n")
+    }
+    z.unpack <- x.unpack * parscale
+
+    # 2. pack (apply equality constraints)
+    if(lavmodel@eq.constraints) {
+        z.pack <- as.numeric( (z.unpack - lavmodel@eq.constraints.k0) %*%
+                              lavmodel@eq.constraints.K )
+    } else {
+        z.pack <- z.unpack
+    }
+
+    # 3. transform (already constrained) variances to standard deviations?
+    # TODO
+    #if(lavoptions$optim.var.transform == "sqrt" &&
+    #       length(lavmodel@x.free.var.idx) > 0L) {
+    #    # transforming variances using atan (or another sigmoid function?)
+    #    # FIXME: better approach?
+    #    #start.x[lavmodel@x.free.var.idx] <-
+    #    #    atan(start.x[lavmodel@x.free.var.idx])
+    #    start.x[lavmodel@x.free.var.idx] <-
+    #        sqrt(start.x[lavmodel@x.free.var.idx]) # assuming positive var
+    #}
+
+    # final starting values for optimizer
+    start.x <- z.pack
+    if(debug) {
+        cat("start.x = ", start.x, "\n")
+    }
+
+
+    # bounds? (new in 0.6-2)
+    if(is.null(lavpartable$lower)) {
+        lower <- -Inf
+    } else {
+        lower <- lavpartable$lower[ lavpartable$free > 0 ]
+        if(lavmodel@eq.constraints) {
+            # collect non -Inf elements
+            not.inf.val <- lower[is.finite(lower)]
+
+            lowerb <- lower
+            lowerb[lower == -Inf] <- -999999
+
+            # pack
+            l.pack <- as.numeric( (lowerb - lavmodel@eq.constraints.k0) %*%
+                                  lavmodel@eq.constraints.K )
+            lower <- l.pack
+            lower[!(lower %in% not.inf.val)] <- -Inf
+        }
+    }
+    if(is.null(lavpartable$upper)) {
+        upper <- +Inf
+    } else {
+        upper <- lavpartable$upper[ lavpartable$free > 0 ]
+        if(lavmodel@eq.constraints) {
+            # collect non -Inf elements
+            not.inf.val <- upper[is.finite(upper)]
+
+            upperb <- upper
+            upperb[upper == Inf] <- 999999
+
+            # pack
+            l.pack <- as.numeric( (upperb - lavmodel@eq.constraints.k0) %*%
+                                  lavmodel@eq.constraints.K )
+            upper <- l.pack
+            upper[!(upper %in% not.inf.val)] <- Inf
+        }
+    }
+
+
+
+
+    # function to be minimized
+    objective_function <- function(x, verbose = FALSE, infToMax = FALSE) {
+
+        # 3. standard deviations to variances
+        # WARNING: x is still packed here!
+        #if(lavoptions$optim.var.transform == "sqrt" &&
+        #   length(lavmodel@x.free.var.idx) > 0L) {
+        #    #x[lavmodel@x.free.var.idx] <- tan(x[lavmodel@x.free.var.idx])
+        #    x.var <- x[lavmodel@x.free.var.idx]
+        #    x.var.sign <- sign(x.var)
+        #    x[lavmodel@x.free.var.idx] <- x.var.sign * (x.var * x.var) # square!
         #}
 
-        # transform variances back
-        #x[lavmodel@x.free.var.idx] <- tan(x[lavmodel@x.free.var.idx])
-
-        # update GLIST (change `state') and make a COPY!
+        # 2. unpack
         if(lavmodel@eq.constraints) {
-            x <- as.numeric(lavmodel@eq.constraints.K %*% x) + 
+            x <- as.numeric(lavmodel@eq.constraints.K %*% x) +
                             lavmodel@eq.constraints.k0
         }
-        GLIST <- lav_model_x2GLIST(lavmodel, x=x)
 
-        fx <- lav_model_objective(lavmodel       = lavmodel, 
-                                  GLIST          = GLIST, 
-                                  lavsamplestats = lavsamplestats, 
+        # 1. unscale
+        x <- x / parscale
+
+        # update GLIST (change `state') and make a COPY!
+        GLIST <- lav_model_x2GLIST(lavmodel, x = x)
+
+        fx <- lav_model_objective(lavmodel       = lavmodel,
+                                  GLIST          = GLIST,
+                                  lavsamplestats = lavsamplestats,
                                   lavdata        = lavdata,
                                   lavcache       = lavcache,
                                   verbose        = verbose,
-                                  forcePD        = forcePD)
+                                  forcePD        = FALSE)
 
         # only for PML: divide by N (to speed up convergence)
         if(estimator == "PML") {
@@ -62,8 +216,8 @@ lav_model_estimate <- function(lavmodel       = NULL,
 
 
 
-        if(debug || verbose) { 
-            cat("Objective function  = ", sprintf("%18.16f", fx), "\n", sep="") 
+        if(debug || verbose) {
+            cat("Objective function  = ", sprintf("%18.16f", fx), "\n", sep="")
         }
         if(debug) {
             #cat("Current unconstrained parameter values =\n")
@@ -77,29 +231,43 @@ lav_model_estimate <- function(lavmodel       = NULL,
         }
 
         # for L-BFGS-B
-        if(infToMax && is.infinite(fx)) fx <- 1e20
+        #if(infToMax && is.infinite(fx)) fx <- 1e20
+        if(!is.finite(fx)) {
+            fx <- 1e20
+        }
 
         fx
     }
 
-    first.derivative.param <- function(x, verbose=FALSE, infToMax=FALSE) {
+    gradient_function <- function(x, verbose = FALSE, infToMax = FALSE) {
 
         # transform variances back
-        #x[lavmodel@x.free.var.idx] <- tan(x[lavmodel@x.free.var.idx])
+        #if(lavoptions$optim.var.transform == "sqrt" &&
+        #   length(lavmodel@x.free.var.idx) > 0L) {
+        #    #x[lavmodel@x.free.var.idx] <- tan(x[lavmodel@x.free.var.idx])
+        #    x.var <- x[lavmodel@x.free.var.idx]
+        #    x.var.sign <- sign(x.var)
+        #    x[lavmodel@x.free.var.idx] <- x.var.sign * (x.var * x.var) # square!
+        #}
 
-        # update GLIST (change `state') and make a COPY!
+        # 2. unpack
         if(lavmodel@eq.constraints) {
             x <- as.numeric(lavmodel@eq.constraints.K %*% x) +
                             lavmodel@eq.constraints.k0
         }
-        GLIST <- lav_model_x2GLIST(lavmodel, x=x)
 
-        dx <- lav_model_gradient(lavmodel       = lavmodel, 
-                                 GLIST          = GLIST, 
+        # 1. unscale
+        x <- x / parscale
+
+        # update GLIST (change `state') and make a COPY!
+        GLIST <- lav_model_x2GLIST(lavmodel, x = x)
+
+        dx <- lav_model_gradient(lavmodel       = lavmodel,
+                                 GLIST          = GLIST,
                                  lavsamplestats = lavsamplestats,
                                  lavdata        = lavdata,
                                  lavcache       = lavcache,
-                                 type           = "free", 
+                                 type           = "free",
                                  group.weight   = group.weight, ### check me!!
                                  verbose        = verbose,
                                  forcePD        = TRUE)
@@ -108,14 +276,25 @@ lav_model_estimate <- function(lavmodel       = NULL,
             cat("Gradient function (analytical) =\n"); print(dx); cat("\n")
         }
 
-        #print( dx %*% lavmodel@eq.constraints.K )
-        #stop("for now")
+        # 1. scale (note: divide, not multiply!)
+        dx <- dx / parscale
 
-        # handle linear equality constraints
+        # 2. pack
         if(lavmodel@eq.constraints) {
             dx <- as.numeric( dx %*% lavmodel@eq.constraints.K )
         }
 
+        # 3. transform variances back
+        #if(lavoptions$optim.var.transform == "sqrt" &&
+        #   length(lavmodel@x.free.var.idx) > 0L) {
+        #    x.var <- x[lavmodel@x.free.var.idx] # here in 'var' metric
+        #    x.var.sign <- sign(x.var)
+        #    x.var <- abs(x.var)
+        #    x.sd <- sqrt(x.var)
+        #    dx[lavmodel@x.free.var.idx] <-
+        #        ( 2 * x.var.sign * dx[lavmodel@x.free.var.idx] * x.sd )
+        #}
+
         # only for PML: divide by N (to speed up convergence)
         if(estimator == "PML") {
             dx <- dx / lavsamplestats@ntotal
@@ -126,51 +305,40 @@ lav_model_estimate <- function(lavmodel       = NULL,
         }
 
         dx
-    } 
+    }
 
-    first.derivative.param.numerical <- function(x, verbose=FALSE) {
+    gradient_function_numerical <- function(x, verbose=FALSE) {
 
-        # transform variances back
-        #x[lavmodel@x.free.var.idx] <- tan(x[lavmodel@x.free.var.idx])
+        # NOTE: no need to 'tranform' anything here (var/eq)
+        # this is done anyway in objective_function
 
         # numerical approximation using the Richardson method
         npar <- length(x)
         h <- 10e-6
         dx <- numeric( npar )
- 
+
         ## FIXME: call lav_model_objective directly!!
         for(i in 1:npar) {
             x.left <- x.left2 <- x.right <- x.right2 <- x
             x.left[i]  <- x[i] - h; x.left2[i]  <- x[i] - 2*h
             x.right[i] <- x[i] + h; x.right2[i] <- x[i] + 2*h
-            fx.left   <- minimize.this.function(x.left)
-            fx.left2  <- minimize.this.function(x.left2)
-            fx.right  <- minimize.this.function(x.right)
-            fx.right2 <- minimize.this.function(x.right2)
+            fx.left   <- objective_function(x.left)
+            fx.left2  <- objective_function(x.left2)
+            fx.right  <- objective_function(x.right)
+            fx.right2 <- objective_function(x.right2)
             dx[i] <- (fx.left2 - 8*fx.left + 8*fx.right - fx.right2)/(12*h)
         }
 
-        #dx <- lavGradientC(func=minimize.this.function, x=x)
+        #dx <- lavGradientC(func=objective_function, x=x)
         # does not work if pnorm is involved... (eg PML)
 
         if(debug) {
             cat("Gradient function (numerical) =\n"); print(dx); cat("\n")
-        }        
+        }
 
         dx
     }
- 
-    # starting values
-    start.x <- lav_model_get_parameters(lavmodel)
-    if(lavmodel@eq.constraints) {
-        start.x <- as.numeric( (start.x - lavmodel@eq.constraints.k0) %*% 
-                                lavmodel@eq.constraints.K )
-    }
 
-    if(debug) {
-        #cat("start.unco = ", lav_model_get_parameters(lavmodel, type="unco"), "\n")
-        cat("start.x = ", start.x, "\n")
-    }
 
     # check if the initial values produce a positive definite Sigma
     # to begin with -- but only for estimator="ML"
@@ -179,11 +347,10 @@ lav_model_estimate <- function(lavmodel       = NULL,
         Sigma.hat <- computeSigmaHat(lavmodel, extra=TRUE, debug=lavoptions$debug)
         for(g in 1:ngroups) {
             if(!attr(Sigma.hat[[g]], "po")) {
-                group.txt <- ifelse(ngroups > 1, 
+                group.txt <- ifelse(ngroups > 1,
                                     paste(" in group ",g,".",sep=""), ".")
                 if(debug) print(Sigma.hat[[g]])
                 stop("lavaan ERROR: initial model-implied matrix (Sigma) is not positive definite;\n  check your model and/or starting parameters", group.txt)
-                # FIXME: should we stop here?? or try anyway?
                 x <- start.x
                 fx <- as.numeric(NA)
                 attr(fx, "fx.group") <- rep(as.numeric(NA), ngroups)
@@ -196,24 +363,22 @@ lav_model_estimate <- function(lavmodel       = NULL,
         }
     }
 
-    # scaling factors
+
+    # parameter scaling
     # FIXME: what is the best way to set the scale??
     # current strategy: if startx > 1.0, we rescale by using
     # 1/startx
     SCALE <- rep(1.0, length(start.x))
-    #idx <- which(abs(start.x) > 10.0)
-    idx <- which(abs(start.x) > 1.0)
-    if(length(idx) > 0L) SCALE[idx] <- abs(1.0/start.x[idx])
-    #idx <- which(abs(start.x) < 1.0 & start.x != 0.0)
-    #if(length(idx) > 0L) SCALE[idx] <- abs(1.0/start.x[idx])
+    if(lavoptions$optim.parscale == "none") {
+        idx <- which(abs(start.x) > 1.0)
+        if(length(idx) > 0L) {
+            SCALE[idx] <- abs(1.0/start.x[idx])
+        }
+    }
     if(debug) {
         cat("SCALE = ", SCALE, "\n")
     }
 
-    # transforming variances using atan (or another sigmoid function?)
-    # FIXME: better approach?
-    #start.x[lavmodel@x.free.var.idx] <- atan(start.x[lavmodel@x.free.var.idx])
-
 
     # first some nelder mead steps? (default = FALSE)
     INIT_NELDER_MEAD <- lavoptions$optim.init_nelder_mead
@@ -221,9 +386,9 @@ lav_model_estimate <- function(lavmodel       = NULL,
     # gradient: analytic, numerical or NULL?
     if(is.character(lavoptions$optim.gradient)) {
         if(lavoptions$optim.gradient %in% c("analytic","analytical")) {
-            GRADIENT <- first.derivative.param
+            GRADIENT <- gradient_function
         } else if(lavoptions$optim.gradient %in% c("numerical", "numeric")) {
-            GRADIENT <- first.derivative.param.numerical
+            GRADIENT <- gradient_function_numerical
         } else if(lavoptions$optim.gradient %in% c("NULL", "null")) {
             GRADIENT <- NULL
         } else {
@@ -231,16 +396,16 @@ lav_model_estimate <- function(lavmodel       = NULL,
         }
     } else if(is.logical(lavoptions$optim.gradient)) {
         if(lavoptions$optim.gradient) {
-            GRADIENT <- first.derivative.param
+            GRADIENT <- gradient_function
         } else {
             GRADIENT <- NULL
         }
     } else if(is.null(lavoptions$optim.gradient)) {
-        GRADIENT <- first.derivative.param
+        GRADIENT <- gradient_function
     }
 
 
-    # optimizer
+    # default optimizer
     if(length(lavmodel@ceq.nonlinear.idx) == 0L &&
        length(lavmodel@cin.linear.idx)    == 0L &&
        length(lavmodel@cin.nonlinear.idx) == 0L) {
@@ -250,7 +415,11 @@ lav_model_estimate <- function(lavmodel       = NULL,
             #OPTIMIZER <- "L-BFGS-B"  # trouble with Inf values for fx!
         } else {
             OPTIMIZER <- toupper(lavoptions$optim.method)
-            stopifnot(OPTIMIZER %in% c("NLMINB", "BFGS", "L-BFGS-B", "NONE"))
+            stopifnot(OPTIMIZER %in% c("NLMINB0", "NLMINB1", "NLMINB2",
+                      "NLMINB", "BFGS", "L-BFGS-B", "NONE"))
+            if(OPTIMIZER == "NLMINB1") {
+                OPTIMIZER <- "NLMINB"
+            }
         }
     } else {
         OPTIMIZER <- "NLMINB.CONSTR"
@@ -260,9 +429,9 @@ lav_model_estimate <- function(lavmodel       = NULL,
         if(verbose) cat("Initial Nelder-Mead step:\n")
         trace <- 0L; if(verbose) trace <- 1L
         optim.out <- optim(par=start.x,
-                           fn=minimize.this.function,
+                           fn=objective_function,
                            method="Nelder-Mead",
-                           #control=list(maxit=10L, 
+                           #control=list(maxit=10L,
                            #             parscale=SCALE,
                            #             trace=trace),
                            hessian=FALSE,
@@ -271,7 +440,52 @@ lav_model_estimate <- function(lavmodel       = NULL,
         start.x <- optim.out$par
     }
 
-    if(OPTIMIZER == "NLMINB") {
+
+
+    if(OPTIMIZER == "NLMINB0") {
+        if(verbose) cat("Quasi-Newton steps using NLMINB0 (no analytic gradient):\n")
+        #if(debug) control$trace <- 1L;
+        control.nlminb <- list(eval.max=20000L,
+                               iter.max=10000L,
+                               trace=0L,
+                               #abs.tol=1e-20, ### important!! fx never negative
+                               abs.tol=(.Machine$double.eps * 10),
+                               rel.tol=1e-10,
+                               #step.min=2.2e-14, # in =< 0.5-12
+                               step.min=1.0, # 1.0 in < 0.5-21
+                               step.max=1.0,
+                               x.tol=1.5e-8,
+                               xf.tol=2.2e-14)
+        control.nlminb <- modifyList(control.nlminb, lavoptions$control)
+        control <- control.nlminb[c("eval.max", "iter.max", "trace",
+                                    "step.min", "step.max",
+                                    "abs.tol", "rel.tol", "x.tol", "xf.tol")]
+        #cat("DEBUG: control = "); print(str(control.nlminb)); cat("\n")
+        optim.out <- nlminb(start=start.x,
+                            objective=objective_function,
+                            gradient=NULL,
+                            lower=lower,
+                            upper=upper,
+                            control=control,
+                            scale=SCALE,
+                            verbose=verbose)
+        if(verbose) {
+            cat("convergence status (0=ok): ", optim.out$convergence, "\n")
+            cat("nlminb message says: ", optim.out$message, "\n")
+            cat("number of iterations: ", optim.out$iterations, "\n")
+            cat("number of function evaluations [objective, gradient]: ",
+                optim.out$evaluations, "\n")
+        }
+
+        iterations <- optim.out$iterations
+        x          <- optim.out$par
+        if(optim.out$convergence == 0) {
+            converged <- TRUE
+        } else {
+            converged <- FALSE
+        }
+
+    } else if(OPTIMIZER == "NLMINB") {
         if(verbose) cat("Quasi-Newton steps using NLMINB:\n")
         #if(debug) control$trace <- 1L;
         control.nlminb <- list(eval.max=20000L,
@@ -286,16 +500,18 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                x.tol=1.5e-8,
                                xf.tol=2.2e-14)
         control.nlminb <- modifyList(control.nlminb, lavoptions$control)
-        control <- control.nlminb[c("eval.max", "iter.max", "trace", 
+        control <- control.nlminb[c("eval.max", "iter.max", "trace",
                                     "step.min", "step.max",
                                     "abs.tol", "rel.tol", "x.tol", "xf.tol")]
         #cat("DEBUG: control = "); print(str(control.nlminb)); cat("\n")
         optim.out <- nlminb(start=start.x,
-                            objective=minimize.this.function,
+                            objective=objective_function,
                             gradient=GRADIENT,
+                            lower=lower,
+                            upper=upper,
                             control=control,
                             scale=SCALE,
-                            verbose=verbose) 
+                            verbose=verbose)
         if(verbose) {
             cat("convergence status (0=ok): ", optim.out$convergence, "\n")
             cat("nlminb message says: ", optim.out$message, "\n")
@@ -317,7 +533,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
         # (but WLS works!)
         # - BB.ML works too
 
-        control.bfgs <- list(trace=0L, fnscale=1, 
+        control.bfgs <- list(trace=0L, fnscale=1,
                              parscale=SCALE, ## or not?
                              ndeps=1e-3,
                              maxit=10000,
@@ -329,7 +545,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                   "maxit", "abstol", "reltol", "REPORT")]
         #trace <- 0L; if(verbose) trace <- 1L
         optim.out <- optim(par=start.x,
-                           fn=minimize.this.function,
+                           fn=objective_function,
                            gr=GRADIENT,
                            method="BFGS",
                            control=control,
@@ -364,13 +580,15 @@ lav_model_estimate <- function(lavmodel       = NULL,
                                factr=1e7,
                                pgtol=0)
         control.lbfgsb <- modifyList(control.lbfgsb, lavoptions$control)
-        control <- control.lbfgsb[c("trace", "fnscale", "parscale", 
-                                    "ndeps", "maxit", "REPORT", "lmm", 
+        control <- control.lbfgsb[c("trace", "fnscale", "parscale",
+                                    "ndeps", "maxit", "REPORT", "lmm",
                                     "factr", "pgtol")]
         optim.out <- optim(par=start.x,
-                           fn=minimize.this.function,
+                           fn=objective_function,
                            gr=GRADIENT,
                            method="L-BFGS-B",
+                           lower=lower,
+                           upper=upper,
                            control=control,
                            hessian=FALSE,
                            verbose=verbose,
@@ -413,7 +631,7 @@ lav_model_estimate <- function(lavmodel       = NULL,
         if(!is.null(body(lavmodel@ceq.jacobian))) ceq.jac <- lavmodel@ceq.jacobian
         trace <- FALSE; if(verbose) trace <- TRUE
         optim.out <- nlminb.constr(start = start.x,
-                                   objective=minimize.this.function,
+                                   objective=objective_function,
                                    gradient=GRADIENT,
                                    control=control,
                                    scale=SCALE,
@@ -446,20 +664,32 @@ lav_model_estimate <- function(lavmodel       = NULL,
         optim.out <- list()
     }
 
-    fx <- minimize.this.function(x) # to get "fx.group" attribute
+    fx <- objective_function(x) # to get "fx.group" attribute
+
     # transform back
+    # 3.
+    #if(lavoptions$optim.var.transform == "sqrt" &&
+    #       length(lavmodel@x.free.var.idx) > 0L) {
+    #    #x[lavmodel@x.free.var.idx] <- tan(x[lavmodel@x.free.var.idx])
+    #    x.var <- x[lavmodel@x.free.var.idx]
+    #    x.var.sign <- sign(x.var)
+    #    x[lavmodel@x.free.var.idx] <- x.var.sign * (x.var * x.var) # square!
+    #}
+
+    # 2. unpack
     if(lavmodel@eq.constraints) {
         x <- as.numeric(lavmodel@eq.constraints.K %*% x) +
                         lavmodel@eq.constraints.k0
     }
 
-    # transform variances back
-    #x[lavmodel@x.free.var.idx] <- tan(x[lavmodel@x.free.var.idx])
+    # 1. unscale
+    x <- x / parscale
 
     attr(x, "converged")  <- converged
     attr(x, "iterations") <- iterations
     attr(x, "control")    <- control
     attr(x, "fx")         <- fx
+    attr(x, "parscale")   <- parscale
     if(!is.null(optim.out$con.jac)) attr(x, "con.jac")    <- optim.out$con.jac
     if(!is.null(optim.out$lambda))  attr(x, "con.lambda") <- optim.out$lambda
     if(lavoptions$partrace) {
