@@ -1,7 +1,12 @@
 # bootstrap based NVCOV
-lav_model_nvcov_bootstrap <- function(lavmodel = NULL, lavsamplestats = NULL, 
-                                      lavoptions = NULL, lavdata = NULL,
-                                      lavcache = NULL, lavpartable = NULL) {
+lav_model_nvcov_bootstrap <- function(lavmodel       = NULL,
+                                      lavsamplestats = NULL,
+                                      lavoptions     = NULL,
+                                      lavimplied     = NULL,
+                                      lavh1          = NULL,
+                                      lavdata        = NULL,
+                                      lavcache       = NULL,
+                                      lavpartable    = NULL) {
 
     # number of bootstrap draws
     if(!is.null(lavoptions$bootstrap)) {
@@ -9,18 +14,18 @@ lav_model_nvcov_bootstrap <- function(lavmodel = NULL, lavsamplestats = NULL,
     } else {
         R <- 1000L
     }
-  
+
     boot.type <- "ordinary"
     if(lavoptions$test == "bollen.stine") boot.type <- "bollen.stine"
 
     TEST <- NULL
     COEF <- bootstrap.internal(object          = NULL,
-                               lavmodel.       = lavmodel, 
-                               lavsamplestats. = lavsamplestats, 
-                               lavpartable.    = lavpartable, 
-                               lavoptions.     = lavoptions, 
+                               lavmodel.       = lavmodel,
+                               lavsamplestats. = lavsamplestats,
+                               lavpartable.    = lavpartable,
+                               lavoptions.     = lavoptions,
                                lavdata.        = lavdata,
-                               R               = R, 
+                               R               = R,
                                verbose         = lavoptions$verbose,
                                type            = boot.type,
                                FUN  = ifelse(boot.type == "bollen.stine",
@@ -39,43 +44,50 @@ lav_model_nvcov_bootstrap <- function(lavmodel = NULL, lavsamplestats = NULL,
     # save COEF and TEST (if any)
     attr(NVarCov, "BOOT.COEF") <- COEF
     attr(NVarCov, "BOOT.TEST") <- TEST
- 
+
     NVarCov
 }
 
 
 # robust `sem' NVCOV (see Browne, 1984,  bentler & dijkstra 1985)
-lav_model_nvcov_robust_sem <- function(lavmodel = NULL, lavsamplestats = NULL,
-                                       lavdata = NULL, lavcache = NULL,
-                                       mimic = "lavaan",
-                                       use.ginv = FALSE) {
+lav_model_nvcov_robust_sem <- function(lavmodel       = NULL,
+                                       lavsamplestats = NULL,
+                                       lavdata        = NULL,
+                                       lavcache       = NULL,
+                                       lavimplied     = NULL,
+                                       lavh1          = NULL,
+                                       lavoptions     = NULL,
+                                       use.ginv       = FALSE) {
 
     # compute inverse of the expected(!) information matrix
-    if(lavmodel@estimator == "ML" && mimic == "Mplus") {
+    if(lavmodel@estimator == "ML" && lavoptions$mimic == "Mplus") {
         # YR - 11 aug 2010 - what Mplus seems to do is (see Muthen apx 4 eq102)
-        # - WLS.V is not based on Sigma.hat and Mu.hat (as it
-        #   should be?), but on lavsamplestats@cov and lavsamplestats@mean...
+        # - A1 is not based on Sigma.hat and Mu.hat,
+        # but on lavsamplestats@cov and lavsamplestats@mean... ('unstructured')
         # - Gamma is not identical to what is used for WLS; closer to EQS
         # - N/N-1 bug in G11 for NVarCov (but not test statistic)
         # - we divide by N-1! (just like EQS)
-        E.inv <- lav_model_information_expected_MLM(lavmodel = lavmodel, 
+        E.inv <- lav_model_information_expected_MLM(lavmodel = lavmodel,
                                            lavsamplestats = lavsamplestats,
                                            extra          = TRUE,
                                            augmented      = TRUE,
                                            inverted       = TRUE,
                                            use.ginv       = use.ginv)
     } else {
-        E.inv <- lav_model_information_expected(lavmodel = lavmodel, 
-                                        lavsamplestats = lavsamplestats, 
-                                        lavdata        = lavdata, 
-                                        extra          = TRUE,
-                                        augmented      = TRUE,
-                                        inverted       = TRUE,
-                                        use.ginv       = use.ginv)
+        E.inv <- lav_model_information(lavmodel       = lavmodel,
+                                       lavsamplestats = lavsamplestats,
+                                       lavdata        = lavdata,
+                                       lavimplied     = lavimplied,
+                                       lavh1          = lavh1,
+                                       lavoptions     = lavoptions,
+                                       extra          = TRUE,
+                                       augmented      = TRUE,
+                                       inverted       = TRUE,
+                                       use.ginv       = use.ginv)
     }
 
     # check if E.inv is ok
-    if(inherits(E.inv, "try-error")) { 
+    if(inherits(E.inv, "try-error")) {
         return(E.inv)
     }
 
@@ -84,9 +96,9 @@ lav_model_nvcov_robust_sem <- function(lavmodel = NULL, lavsamplestats = NULL,
 
     # Gamma
     Gamma <- lavsamplestats@NACOV
-    if(lavmodel@estimator == "ML" && 
-       mimic == "Mplus" && !lavsamplestats@NACOV.user) {
-        # 'fix' G11 part of Gamma (NOTE: this is NOT needed for SB test 
+    if(lavmodel@estimator == "ML" &&
+       lavoptions$mimic == "Mplus" && !lavsamplestats@NACOV.user) {
+        # 'fix' G11 part of Gamma (NOTE: this is NOT needed for SB test
         # statistic
         for(g in 1:lavsamplestats@ngroups) {
             gg1 <- (lavsamplestats@nobs[[g]]-1)/lavsamplestats@nobs[[g]]
@@ -99,12 +111,18 @@ lav_model_nvcov_robust_sem <- function(lavmodel = NULL, lavsamplestats = NULL,
             Gamma[[g]][1:nvar, 1:nvar] <- G11 * gg1
         } # g
     }
-   
+
 
     tDVGVD <- matrix(0, ncol=ncol(E.inv), nrow=nrow(E.inv))
     for(g in 1:lavsamplestats@ngroups) {
         fg  <-  lavsamplestats@nobs[[g]]   /lavsamplestats@ntotal
-        fg1 <- (lavsamplestats@nobs[[g]]-1)/lavsamplestats@ntotal
+        if(lavoptions$mimic == "Mplus") {
+            fg1 <- (lavsamplestats@nobs[[g]]-1)/lavsamplestats@ntotal
+        } else {
+            # from 0.6 onwards, we use fg1 == fg, to be more consistent with
+            # lav_test()
+            fg1 <- fg
+        }
         # fg twice for WLS.V, 1/fg1 once for GaMMA
         # if fg==fg1, there would be only one fg, as in Satorra 1999 p.8
         # t(Delta) * WLS.V %*% Gamma %*% WLS.V %*% Delta
@@ -119,7 +137,7 @@ lav_model_nvcov_robust_sem <- function(lavmodel = NULL, lavsamplestats = NULL,
     } # g
     NVarCov <- (E.inv %*% tDVGVD %*% E.inv)
 
-    # to be reused by lavaanTest
+    # to be reused by lav_test()
     attr(NVarCov, "E.inv") <- E.inv
     attr(NVarCov, "Delta") <- Delta
     attr(NVarCov, "WLS.V") <- WLS.V
@@ -127,11 +145,13 @@ lav_model_nvcov_robust_sem <- function(lavmodel = NULL, lavsamplestats = NULL,
     NVarCov
 }
 
-lav_model_nvcov_robust_sandwich <- function(lavmodel      = lavmodel, 
-                                            lavsamplestats = NULL, 
+lav_model_nvcov_robust_sandwich <- function(lavmodel       = NULL,
+                                            lavsamplestats = NULL,
                                             lavdata        = NULL,
-                                            information    = "observed", 
-                                            lavcache       = NULL, 
+                                            lavoptions     = NULL,
+                                            lavimplied     = NULL,
+                                            lavh1          = NULL,
+                                            lavcache       = NULL,
                                             use.ginv       = FALSE) {
 
     # sandwich estimator: A.inv %*% B %*% t(A.inv)
@@ -143,7 +163,9 @@ lav_model_nvcov_robust_sandwich <- function(lavmodel      = lavmodel,
                                    lavsamplestats = lavsamplestats,
                                    lavdata        = lavdata,
                                    lavcache       = lavcache,
-                                   information    = information,
+                                   lavimplied     = lavimplied,
+                                   lavh1          = lavh1,
+                                   lavoptions     = lavoptions,
                                    extra          = FALSE,
                                    augmented      = TRUE,
                                    inverted       = TRUE,
@@ -155,11 +177,14 @@ lav_model_nvcov_robust_sandwich <- function(lavmodel      = lavmodel,
     }
 
     # outer product of case-wise scores
-    B0 <- 
+    B0 <-
         lav_model_information_firstorder(lavmodel       = lavmodel,
                                          lavsamplestats = lavsamplestats,
                                          lavdata        = lavdata,
                                          lavcache       = lavcache,
+                                         lavimplied     = lavimplied,
+                                         lavh1          = lavh1,
+                                         lavoptions     = lavoptions,
                                          extra          = TRUE,
                                          check.pd       = FALSE,
                                          augmented      = FALSE,
@@ -175,10 +200,10 @@ lav_model_nvcov_robust_sandwich <- function(lavmodel      = lavmodel,
     NVarCov
 }
 
-# two stage 
+# two stage
 # - two.stage: Gamma = I_1^{-1}
 # - robust.two.stage: Gamma = incomplete Gamma (I_1^{-1} J_1 I_1^{-1})
-# where I_1 and J_1 are based on the (saturated) model h1 
+# where I_1 and J_1 are based on the (saturated) model h1
 # (either unstructured, or structured)
 #
 # references:
@@ -186,70 +211,67 @@ lav_model_nvcov_robust_sandwich <- function(lavmodel      = lavmodel,
 # - Savalei \& Bentler (2009) eq (6) for se = "two.stage"
 # - Savalei \& Falk (2014) eq  (3)   for se = "robust.two.stage"
 # - Yuan \& Bentler (2000)
-lav_model_nvcov_two_stage <- function(lavmodel       = NULL, 
+lav_model_nvcov_two_stage <- function(lavmodel       = NULL,
                                       lavsamplestats = NULL,
                                       lavoptions     = NULL,
-                                      lavdata        = NULL,
                                       lavimplied     = NULL,
+                                      lavh1          = NULL,
+                                      lavdata        = NULL,
                                       use.ginv       = FALSE) {
 
-
     # expected OR observed, depending on lavoptions$information
     if(is.null(lavoptions) && is.null(lavoptions$information)) {
-        lavoptions <- list(information = "observed")
+        lavoptions <- list(information = "observed",
+                           observed.information = "h1",
+                           h1.information = "structured")
+    }
+
+    # restrictions:
+    # only works if:
+    # - information is expected,
+    # - or information is observed but with observed.information == "h1"
+    if(lavoptions$information == "observed" &&
+       lavoptions$observed.information != "h1") {
+            stop("lavaan ERROR: two.stage + observed information currently only works with observed.information = ", dQuote("h1"))
+    }
+    # no weights (yet)
+    if(!is.null(lavdata@weights[[1]])) {
+        stop("lavaan ERROR: two.stage + sampling.weights is not supported yet")
+    }
+    # no fixed.x (yet)
+    if(!is.null(lavsamplestats@x.idx) &&
+       length(lavsamplestats@x.idx[[1]]) > 0L) {
+        stop("lavaan ERROR: two.stage + fixed.x = TRUE is not supported yet")
     }
 
-    
 
     # information matrix
-    if(lavoptions$information == "expected") {
-        # structured of unstructured?
-        if(!is.null(lavoptions) &&
-           !is.null(lavoptions$h1.information) &&
-           lavoptions$h1.information == "unstructured") {
-            structured <- FALSE
-        } else {
-            structured <- TRUE
-        }
-
-        E.inv <- lav_model_information_expected(lavmodel       = lavmodel,
-                                                lavsamplestats = lavsamplestats,
-                                                structured     = structured,
-                                                lavdata        = NULL,
-                                                extra          = TRUE,
-                                                augmented      = TRUE,
-                                                inverted       = TRUE,
-                                                use.ginv       = use.ginv)
-        Delta <- attr(E.inv, "Delta")
-        WLS.V <- attr(E.inv, "WLS.V") # this is 'H' in the literature
-        attr(E.inv, "Delta") <- NULL
-        attr(E.inv, "WLS.V") <- NULL
-    } else {
-        E.inv <- lav_model_information_observed(lavmodel       = lavmodel,
-                                                lavsamplestats = lavsamplestats,
-                                                lavoptions     = lavoptions,
-                                                lavdata        = NULL,
-                                                augmented      = TRUE,
-                                                inverted       = TRUE,
-                                                use.ginv       = use.ginv)
-        if(lavoptions$observed.information == "h1") {
-            Delta <- attr(E.inv, "Delta")
-            WLS.V <- attr(E.inv, "WLS.V") # this is 'H' in the literature
-            attr(E.inv, "Delta") <- NULL
-            attr(E.inv, "WLS.V") <- NULL
-        } else {
-            stop("lavaan ERROR: two.stage + observed information currently only works with observed.information = ", dQuote("h1"))
-        }
-    }
+    E.inv <- lav_model_information(lavmodel       = lavmodel,
+                                   lavsamplestats = lavsamplestats,
+                                   lavdata        = lavdata,
+                                   lavoptions     = lavoptions,
+                                   lavimplied     = lavimplied,
+                                   lavh1          = lavh1,
+                                   extra          = TRUE,
+                                   augmented      = TRUE,
+                                   inverted       = TRUE,
+                                   use.ginv       = use.ginv)
+    Delta <- attr(E.inv, "Delta")
+    WLS.V <- attr(E.inv, "WLS.V") # this is 'H' or 'A1' in the literature
+    attr(E.inv, "Delta") <- NULL
+    attr(E.inv, "WLS.V") <- NULL
 
     # check if E.inv is ok
     if(inherits(E.inv, "try-error")) {
         return(E.inv)
     }
 
+    # check WLS.V = A1
     if(is.null(WLS.V)) {
-        stop("lavaan ERROR: WLS.V/H is NULL, observed.information = hessian?")
+        stop("lavaan ERROR: WLS.V/H/A1 is NULL, observed.information = hessian?")
     }
+
+    # Gamma
     Gamma <- vector("list", length = lavsamplestats@ngroups)
 
     # handle multiple groups
@@ -263,7 +285,7 @@ lav_model_nvcov_two_stage <- function(lavmodel       = NULL,
         # t(Delta) * WLS.V %*% Gamma %*% WLS.V %*% Delta
         WD <- WLS.V[[g]] %*% Delta[[g]]
 
-        # to compute (incomplete) GAMMA, should we use 
+        # to compute (incomplete) GAMMA, should we use
         # structured or unstructured mean/sigma?
         #
         # we use the same setting as to compute 'H' (the h1 information matrix)
@@ -273,7 +295,7 @@ lav_model_nvcov_two_stage <- function(lavmodel       = NULL,
             SIGMA <- lavsamplestats@missing.h1[[g]]$sigma
         } else {
             MU    <- lavimplied$mean[[g]]
-            SIGMA <- lavimplied$cov[[g]]   
+            SIGMA <- lavimplied$cov[[g]]
         }
 
         # compute 'Gamma' (or Omega.beta)
@@ -281,25 +303,37 @@ lav_model_nvcov_two_stage <- function(lavmodel       = NULL,
             # this is Savalei & Bentler (2009)
             if(lavoptions$information == "expected") {
                 Info <- lav_mvnorm_missing_information_expected(
-                            Y = lavdata@X[[g]], Mp = lavdata@Mp[[g]], 
-                            Mu = MU, Sigma = SIGMA)
+                            Y = lavdata@X[[g]], Mp = lavdata@Mp[[g]],
+                            wt = lavdata@weights[[g]],
+                            Mu = MU, Sigma = SIGMA,
+                            x.idx = lavsamplestats@x.idx[[g]])
             } else {
                 Info <- lav_mvnorm_missing_information_observed_samplestats(
-                            Yp = lavsamplestats@missing[[g]], 
-                            Mu = MU, Sigma = SIGMA)
+                            Yp = lavsamplestats@missing[[g]],
+                            # wt not needed
+                            Mu = MU, Sigma = SIGMA,
+                            x.idx = lavsamplestats@x.idx[[g]])
             }
             Gamma[[g]] <- lav_matrix_symmetric_inverse(Info)
         } else { # we assume "robust.two.stage"
                  # NACOV is here incomplete Gamma
                  # Savalei & Falk (2014)
                  #
-            Gamma[[g]] <- lav_mvnorm_missing_h1_omega_sw(Y = 
-                             lavdata@X[[g]], Mp = lavdata@Mp[[g]], 
-                             Yp = lavsamplestats@missing[[g]], 
+            if(length(lavdata@cluster) > 0L) {
+                cluster.idx <- lavdata@Lp[[g]]$cluster.idx[[2]]
+            } else {
+                cluster.idx <- NULL
+            }
+            Gamma[[g]] <- lav_mvnorm_missing_h1_omega_sw(Y =
+                             lavdata@X[[g]], Mp = lavdata@Mp[[g]],
+                             Yp = lavsamplestats@missing[[g]],
+                             wt = lavdata@weights[[g]],
+                             cluster.idx = cluster.idx,
                              Mu = MU, Sigma = SIGMA,
+                             x.idx = lavsamplestats@x.idx[[g]],
                              information = lavoptions$information)
         }
- 
+
         # compute
         tDVGVD <- tDVGVD + fg*fg/fg1 * crossprod(WD, Gamma[[g]] %*% WD)
     } # g
@@ -310,20 +344,23 @@ lav_model_nvcov_two_stage <- function(lavmodel       = NULL,
     attr(NVarCov, "Delta") <- Delta
     attr(NVarCov, "Gamma") <- Gamma
     #if(lavoptions$h1.information.se == lavoptions$h1.information.test) {
-        attr(NVarCov, "E.inv") <- E.inv 
+        attr(NVarCov, "E.inv") <- E.inv
         attr(NVarCov, "WLS.V") <- WLS.V
     #}
 
     NVarCov
 }
 
-lav_model_vcov <- function(lavmodel       = NULL, 
-                           lavsamplestats = NULL, 
-                           lavoptions     = NULL, 
-                           lavdata        = NULL, 
-                           lavpartable    = NULL, 
+
+
+lav_model_vcov <- function(lavmodel       = NULL,
+                           lavsamplestats = NULL,
+                           lavoptions     = NULL,
+                           lavdata        = NULL,
+                           lavpartable    = NULL,
                            lavcache       = NULL,
                            lavimplied     = NULL,
+                           lavh1          = NULL,
                            use.ginv       = FALSE) {
 
     likelihood  <- lavoptions$likelihood
@@ -331,14 +368,10 @@ lav_model_vcov <- function(lavmodel       = NULL,
     se          <- lavoptions$se
     verbose     <- lavoptions$verbose
     mimic       <- lavoptions$mimic
-  
-    # special cases
-    if(se == "none" || se == "external") return(matrix(0,0,0))
 
-    # some require meanstructure (for now)
-    if(se %in% c("first.order", "robust.sem", "robust.huber.white") && 
-       !lavoptions$meanstructure) {
-        stop("se (", se, ") requires meanstructure (for now)")
+    # special cases
+    if(se == "none" || se == "external") {
+        return( matrix(0, 0, 0) )
     }
 
     if(se == "standard") {
@@ -346,40 +379,49 @@ lav_model_vcov <- function(lavmodel       = NULL,
                                          lavsamplestats = lavsamplestats,
                                          lavdata        = lavdata,
                                          lavcache       = lavcache,
-                                         information    = information,
+                                         lavimplied     = lavimplied,
+                                         lavh1          = lavh1,
+                                         lavoptions     = lavoptions,
                                          extra          = FALSE,
                                          augmented      = TRUE,
                                          inverted       = TRUE,
                                          use.ginv       = use.ginv)
 
     } else if(se == "first.order") {
-        NVarCov <- 
+        NVarCov <-
             lav_model_information_firstorder(lavmodel = lavmodel,
                                              lavsamplestats = lavsamplestats,
                                              lavdata        = lavdata,
                                              lavcache       = lavcache,
+                                             lavimplied     = lavimplied,
+                                             lavh1          = lavh1,
+                                             lavoptions     = lavoptions,
                                              extra          = TRUE,
                                              check.pd       = FALSE,
                                              augmented      = TRUE,
                                              inverted       = TRUE,
                                              use.ginv       = use.ginv)
-    
-    } else if(se == "robust.sem") {
+
+    } else if(se == "robust.sem" || se == "robust.cluster.sem") {
         NVarCov <-
             lav_model_nvcov_robust_sem(lavmodel       = lavmodel,
                                        lavsamplestats = lavsamplestats,
-                                       mimic          = mimic,
                                        lavcache       = lavcache,
                                        lavdata        = lavdata,
+                                       lavimplied     = lavimplied,
+                                       lavh1          = lavh1,
+                                       lavoptions     = lavoptions,
                                        use.ginv       = use.ginv)
 
-    } else if(se == "robust.huber.white") {
+    } else if(se == "robust.huber.white" || se == "robust.cluster") {
         NVarCov <-
             lav_model_nvcov_robust_sandwich(lavmodel = lavmodel,
                                             lavsamplestats = lavsamplestats,
                                             lavdata        = lavdata,
-                                            information    = information,
                                             lavcache       = lavcache,
+                                            lavimplied     = lavimplied,
+                                            lavh1          = lavh1,
+                                            lavoptions     = lavoptions,
                                             use.ginv       = use.ginv)
 
     } else if(se %in% c("two.stage", "robust.two.stage")) {
@@ -389,6 +431,7 @@ lav_model_vcov <- function(lavmodel       = NULL,
                                       lavoptions     = lavoptions,
                                       lavdata        = lavdata,
                                       lavimplied     = lavimplied,
+                                      lavh1          = lavh1,
                                       use.ginv       = use.ginv)
 
     } else if(se == "bootstrap") {
@@ -396,6 +439,8 @@ lav_model_vcov <- function(lavmodel       = NULL,
                                         lavsamplestats = lavsamplestats,
                                         lavoptions     = lavoptions,
                                         lavdata        = lavdata,
+                                        lavimplied     = lavimplied,
+                                        lavh1          = lavh1,
                                         lavcache       = lavcache,
                                         lavpartable    = lavpartable),
                         silent=TRUE )
@@ -406,23 +451,97 @@ lav_model_vcov <- function(lavmodel       = NULL,
     if(! inherits(NVarCov, "try-error") ) {
 
         # denominator!
-        if(lavmodel@estimator %in% c("ML","PML","FML") && 
+        if(lavmodel@estimator %in% c("ML","PML","FML") &&
            likelihood == "normal") {
-            N <- lavsamplestats@ntotal
+            if(lavdata@nlevels == 1L) {
+                N <- lavsamplestats@ntotal
+            } else {
+                # total number of clusters (over groups)
+                N <- 0
+                for(g in 1:lavsamplestats@ngroups) {
+                    N <- N + lavdata@Lp[[g]]$nclusters[[2]]
+                }
+            }
         } else {
             N <- lavsamplestats@ntotal - lavsamplestats@ngroups
         }
 
-        #if(lavmodle@estimator %in% c("PML", "MML")) {
-        #    VarCov <- NVarCov
-        #} else {
-            VarCov <- 1/N * NVarCov
-        #}
+        VarCov <- 1/N * NVarCov
+
+        # check if VarCov is pd -- new in 0.6-2
+        # mostly important if we have (in)equality constraints (MASS::ginv!)
+        if(!is.null(lavoptions$check.vcov) && lavoptions$check.vcov) {
+            eigvals <- eigen(VarCov, symmetric = TRUE,
+                             only.values = TRUE)$values
+            # correct for (in)equality constraints
+            neq <- 0L
+            niq <- 0L
+            if(nrow(lavmodel@con.jac) > 0L) {
+                ceq.idx <- attr(lavmodel@con.jac, "ceq.idx")
+                cin.idx <- attr(lavmodel@con.jac, "cin.idx")
+                ina.idx <- attr(lavmodel@con.jac, "inactive.idx")
+                if(length(ceq.idx) > 0L) {
+                    neq <- qr(lavmodel@con.jac[ceq.idx,,drop=FALSE])$rank
+                }
+                if(length(cin.idx) > 0L) {
+                    niq <- length(cin.idx) - length(ina.idx) # only active
+                }
+                # total number of relevant constraints
+                neiq <- neq + niq
+                if(neiq > 0L) {
+                    eigvals <- rev(eigvals)[- seq_len(neiq)]
+                }
+            }
+            min.val <- min(eigvals)
+            #if(any(eigvals < -1 * sqrt(.Machine$double.eps)) &&
+            if(min.val < .Machine$double.eps^(3/4) && lavoptions$warn) {
+                #VarCov.chol <- suppressWarnings(try(chol(VarCov,
+                #                   pivot = TRUE), silent = TRUE))
+                #VarCov.rank <- attr(VarCov.chol, "rank")
+                #VarCov.pivot <- attr(VarCov.chol, "pivot")
+                #VarCov.badidx <- VarCov.pivot[ VarCov.rank + 1L ]
+                #pt.idx <- which(lavpartable$free == VarCov.badidx)
+                #par.string <- paste(lavpartable$lhs[pt.idx],
+                #                    lavpartable$op[ pt.idx],
+                #                    lavpartable$rhs[pt.idx])
+                #if(lavdata@ngroups > 1L) {
+                #    par.string <- paste0(par.string, " in group ",
+                #                         lavpartable$group[pt.idx])
+                #}
+                #if(lavdata@nlevels > 1L) {
+                #    par.string <- paste0(par.string, " in level ",
+                #                         lavpartable$level[pt.idx])
+                #}
+
+                if(min.val > 0) {
+                    txt <- c("The variance-covariance matrix of the estimated
+                        parameters (vcov) does not appear to be positive
+                        definite! The smallest eigenvalue (= ",
+                        sprintf("%e", min(min.val)), ") is close to zero.
+                        This may be a symptom that the model is not
+                        identified.")
+                    warning(lav_txt2message(txt))
+                } else {
+                    txt <- c("The variance-covariance matrix of the estimated
+                        parameters (vcov) does not appear to be positive
+                        definite! The smallest eigenvalue (= ",
+                        sprintf("%e", min(min.val)), ") is smaller than zero.
+                        This may be a symptom that the model is not
+                        identified.")
+                    warning(lav_txt2message(txt))
+                }
+            }
+        }
 
     } else {
-        warning("lavaan WARNING: could not compute standard errors!\n  lavaan NOTE: this may be a symptom that the model is not identified.\n")
+        if(lavoptions$warn) {
+            txt <- "Could not compute standard errors! The information matrix
+                    could not be inverted. This may be a symptom that the model
+                    is not identified."
+            warning(lav_txt2message(txt))
+        }
         VarCov <- NULL
-    }
+    } # could not invert
 
     VarCov
 }
@@ -454,7 +573,7 @@ lav_model_vcov_se <- function(lavmodel, lavpartable, VCOV = NULL,
         se[ which(lavpartable$free == 0L) ] <- 0.0
 
 
-    # 3. defined parameters: 
+    # 3. defined parameters:
         def.idx <- which(lavpartable$op == ":=")
         if(length(def.idx) > 0L) {
             if(!is.null(BOOT)) {
