@@ -1,5 +1,5 @@
 # fitting function for standard ML
-estimator.ML <- function(Sigma.hat=NULL, Mu.hat=NULL, 
+estimator.ML <- function(Sigma.hat=NULL, Mu.hat=NULL,
                          data.cov=NULL, data.mean=NULL,
                          data.cov.log.det=NULL,
                          meanstructure=FALSE) {
@@ -31,7 +31,7 @@ estimator.ML <- function(Sigma.hat=NULL, Mu.hat=NULL,
 # fitting function for standard ML
 estimator.ML_res <- function(Sigma.hat=NULL, Mu.hat=NULL, PI=NULL,
                              res.cov=NULL, res.int=NULL, res.slopes=NULL,
-                             res.cov.log.det=NULL, 
+                             res.cov.log.det=NULL,
                              cov.x = NULL, mean.x = NULL) {
 
     if(!attr(Sigma.hat, "po")) return(Inf)
@@ -67,7 +67,7 @@ estimator.REML <- function(Sigma.hat=NULL, Mu.hat=NULL,
                            data.cov=NULL, data.mean=NULL,
                            data.cov.log.det=NULL,
                            meanstructure=FALSE,
-                           group = 1L, lavmodel = NULL, 
+                           group = 1L, lavmodel = NULL,
                            lavsamplestats = NULL, lavdata = NULL) {
 
     if(!attr(Sigma.hat, "po")) return(Inf)
@@ -134,6 +134,7 @@ estimator.WLS <- function(WLS.est=NULL, WLS.obs=NULL, WLS.V=NULL) {
     # since 0.5-17, we use crossprod twice
     diff <- WLS.obs - WLS.est
     fx <- as.numeric( crossprod(crossprod(WLS.V, diff), diff) )
+    # (faster?) alternative: sum(WLS.V * tcrossprod(diff))
 
     # no negative values
     if(is.finite(fx) && fx < 0.0) fx <- 0.0
@@ -153,8 +154,8 @@ estimator.DWLS <- function(WLS.est = NULL, WLS.obs = NULL, WLS.VD = NULL) {
     fx
 }
 
-# Full Information ML estimator (FIML) handling the missing values 
-estimator.FIML <- function(Sigma.hat = NULL, Mu.hat = NULL, Yp = NULL, 
+# Full Information ML estimator (FIML) handling the missing values
+estimator.FIML <- function(Sigma.hat = NULL, Mu.hat = NULL, Yp = NULL,
                            h1 = NULL, N = NULL) {
 
     if(is.null(N)) {
@@ -182,11 +183,11 @@ estimator.FIML <- function(Sigma.hat = NULL, Mu.hat = NULL, Yp = NULL,
 #
 # some changes:
 # - no distinction between x/y (ksi/eta)
-# - loglikelihoods are computed case-wise
 # - 29/03/2016: adapt for exogenous covariates
-# - 21/09/2016: added code for missing = doubly.robust (contributed by 
+# - 21/09/2016: added code for missing = doubly.robust (contributed by
 #   Myrsini Katsikatsou)
 estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
+                          Mu.hat     = NULL,    # model-based means
                           TH         = NULL,    # model-based thresholds + means
                           PI         = NULL,    # slopes
                           th.idx     = NULL,    # threshold idx per variable
@@ -197,26 +198,39 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
                           missing    = NULL) {  # how to deal with missings?
 
     # YR 3 okt 2012
-    # the idea is to compute for each pair of variables, the model-based 
-    # probability (or likelihood in mixed case) (that we observe the data 
-    # for this pair under the model) for *each case* 
-    # after taking logs, the sum over the cases gives the 
-    # log probablity/likelihood for this pair
-    # the sum over all pairs gives the final PML based logl
+    # - the idea is to compute for each pair of variables, the model-based
+    #   probability (or likelihood in mixed case) (that we observe the data
+    #   for this pair under the model)
+    # - if we have exogenous variables + condidional.x, do this for each case
+    # - after taking logs, the sum over the cases gives the
+    #   log probablity/likelihood for this pair
+    # - the sum over all pairs gives the final PL based logl
 
     # first of all: check if all correlations are within [-1,1]
     # if not, return Inf; (at least with nlminb, this works well)
-    cors <- Sigma.hat[lower.tri(Sigma.hat)]
 
-    #cat("[DEBUG objective\n]"); print(range(cors)); print(range(TH)); cat("\n")
-    if(any(abs(cors) > 1)) {
+    # diagonal of Sigma.hat is not necessarily 1, even for categorical vars
+    Sigma.hat2 <- Sigma.hat
+    if(length(num.idx) > 0L) {
+        diag(Sigma.hat2)[-num.idx] <- 1
+    } else {
+        diag(Sigma.hat2) <- 1
+    }
+    # all positive variances? (for continuous variables)
+    if(any(diag(Sigma.hat2) < 0)) {
+        OUT <- +Inf
+        attr(OUT, "logl") <- as.numeric(NA)
+        return(OUT)
+    }
+    Cor.hat <- cov2cor(Sigma.hat2) # to get correlations (rho!)
+    cors <- lav_matrix_vech(Cor.hat, diagonal = FALSE)
+
+    if(length(cors) > 0L && (any(abs(cors) > 1) ||
+                             any(is.na(cors)))) {
         # question: what is the best approach here??
         OUT <- +Inf
         attr(OUT, "logl") <- as.numeric(NA)
-        return(OUT) 
-        #idx <- which( abs(cors) > 0.99 )
-        #cors[idx] <- 0.99 # clip
-        #cat("CLIPPING!\n")
+        return(OUT)
     }
 
     nvar <- nrow(Sigma.hat)
@@ -227,30 +241,37 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
     }
     pstar <- nvar*(nvar-1)/2
     ov.types <- rep("ordered", nvar)
-    if(length(num.idx) > 0L) ov.types[num.idx] <- "numeric"
-    #print(Sigma.hat); print(TH); print(th.idx); print(num.idx); print(str(X))
+    if(length(num.idx) > 0L) {
+        ov.types[num.idx] <- "numeric"
+    }
 
-    LIK <- matrix(0, nrow(X), pstar) # likelihood per case, per pair
-    PSTAR <- matrix(0, nvar, nvar)   # utility matrix, to get indices
-    PSTAR[lav_matrix_vech_idx(nvar, diagonal = FALSE)] <- 1:pstar
-    PROW <- row(PSTAR)
-    PCOL <- col(PSTAR)
 
-    # shortcut for all ordered - tablewise
+    ##### Three cases:
+    ##### 1) all ordered, no exogenous (fast!)
+    ##### 2) mixed ordered + continuous, no exogenous
+    ##### 3) mixed ordered + continuous, exogenous (conditional.x = TRUE)
+
+
+
+
+    ##### Case 1:
+    #####  all ordered
+    #####  no exogenous covariates
+    #####
     if(all(ov.types == "ordered") && nexo == 0L) {
         # prepare for Myrsini's vectorization scheme
         LONG2 <- LongVecTH.Rho(no.x               = nvar,
                                all.thres          = TH,
-                               index.var.of.thres = th.idx, 
+                               index.var.of.thres = th.idx,
                                rho.xixj           = cors)
         # get expected probability per table, per pair
-        pairwisePI <- pairwiseExpProbVec(ind.vec = lavcache$LONG, 
+        pairwisePI <- pairwiseExpProbVec(ind.vec = lavcache$LONG,
                                          th.rho.vec = LONG2)
         pairwisePI_orig <- pairwisePI # for doubly.robust
 
         # get frequency per table, per pair
         logl <- sum(lavcache$bifreq * log(pairwisePI))
-    
+
         # more convenient fit function
         prop <- lavcache$bifreq / lavcache$nobs
         freq <- lavcache$bifreq
@@ -261,7 +282,7 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
             freq <- freq[-zero.idx]
             prop <- prop[-zero.idx]
             pairwisePI <- pairwisePI[-zero.idx]
-        } 
+        }
         ##Fmin <- sum( prop*log(prop/pairwisePI) )
         Fmin <- sum( freq * log(prop/pairwisePI) ) # to avoid 'N'
 
@@ -278,7 +299,7 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
 
             uniprop <- unifreq / uninobs
 
-            # remove zero props 
+            # remove zero props
             # uni.zero.idx <- which(uniprop == 0.0)
             uni.zero.idx <- which( (uniprop == 0.0) | !is.finite(uniprop) )
             if(length(uni.zero.idx) > 0L) {
@@ -293,39 +314,39 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
         if (missing =="doubly.robust")  {
 
              # COMPUTE THE SUM OF THE EXPECTED BIVARIATE CONDITIONAL LIKELIHOODS
-             #SUM_{i,j} [ E_{Yi,Yj|y^o}}(lnf(Yi,Yj))) ]
+             #SUM_{i,j} [ E_{Yi,Yj|y^o}(lnf(Yi,Yj))) ]
 
              #First compute the terms of the summand. Since the cells of
-             # pairwiseProbGivObs are zero for the pairs of variables that at least 
+             # pairwiseProbGivObs are zero for the pairs of variables that at least
              #one of the variables is observed (hence not contributing to the summand)
-             #there is no need to construct an index vector for summing appropriately 
+             #there is no need to construct an index vector for summing appropriately
              #within each individual.
              log_pairwisePI_orig <- log(pairwisePI_orig)
              pairwiseProbGivObs <- lavcache$pairwiseProbGivObs
              tmp_prod <- t(t(pairwiseProbGivObs)*log_pairwisePI_orig)
-  
+
              SumElnfijCasewise <- apply(tmp_prod, 1, sum)
              SumElnfij <- sum(SumElnfijCasewise)
              logl <- logl + SumElnfij
              Fmin <- Fmin - SumElnfij
 
-             # COMPUTE THE THE SUM OF THE EXPECTED UNIVARIATE CONDITIONAL LIKELIHOODS 
-             # SUM_{i,j} [ E_{Yj|y^o}}(lnf(Yj|yi))) ]
+             # COMPUTE THE THE SUM OF THE EXPECTED UNIVARIATE CONDITIONAL LIKELIHOODS
+             # SUM_{i,j} [ E_{Yj|y^o}(lnf(Yj|yi))) ]
 
              #First compute the model-implied conditional univariate probabilities
              # p(y_i=a|y_j=b). Let ModProbY1Gy2 be the vector of these
              # probabilities. The order the probabilities
-             #are listed in the vector ModProbY1Gy2 is as follows: 
+             #are listed in the vector ModProbY1Gy2 is as follows:
              # y1|y2, y1|y3, ..., y1|yp, y2|y1, y2|y3, ..., y2|yp,
              # ..., yp|y1, yp|y2, ..., yp|y(p-1). Within each pair of variables the
-             #index "a" which represents the response category of variable yi runs faster than 
+             #index "a" which represents the response category of variable yi runs faster than
              #"b" which represents the response category of the given variable yj.
              #The computation of these probabilities are based on the model-implied
              #bivariate probabilities p(y_i=a,y_j=b). To do the appropriate summations
              #and divisions we need some index vectors to keep track of the index i, j,
-             #a, and b, as well as the pair index. These index vectors should be 
+             #a, and b, as well as the pair index. These index vectors should be
              #computed once and stored in lavcache. About where in the lavaan code
-             #we will add the computations and how they will be done please see the 
+             #we will add the computations and how they will be done please see the
              #file "new objects in lavcache for DR-PL.r"
 
              idx.pairs <- lavcache$idx.pairs
@@ -352,14 +373,14 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
 
              log_ModProbY1Gy2 <- log(ModProbY1Gy2)
 
-             #Let univariateProbGivObs be the matrix of the conditional univariate 
+             #Let univariateProbGivObs be the matrix of the conditional univariate
              # probabilities Pr(y_i=a|y^o) that has been computed in advance and are
-             #fed to the DR-PL function. The rows represent different individuals, 
+             #fed to the DR-PL function. The rows represent different individuals,
              #i.e. nrow=nobs, and the columns different probabilities. The columns
-             # are listed as follows: a runs faster than i. 
-   
-             #Note that the number of columns of univariateProbGivObs is not the 
-             #same with the length(log_ModProbY1Gy2), actually 
+             # are listed as follows: a runs faster than i.
+
+             #Note that the number of columns of univariateProbGivObs is not the
+             #same with the length(log_ModProbY1Gy2), actually
              #ncol(univariateProbGivObs) < length(log_ModProbY1Gy2).
              #For this we use the following commands in order to multiply correctly.
 
@@ -371,14 +392,14 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
              id.cases.with.missing <- which(uniweights.casewise > 0)
              no.cases.with.missing <- length(id.cases.with.missing)
              no.obs.casewise <- nvar - uniweights.casewise
-             idx.missing.var <- apply(X, 1, function(x) { 
-                 which(is.na(x)) 
+             idx.missing.var <- apply(X, 1, function(x) {
+                 which(is.na(x))
              })
              idx.observed.var <- lapply(idx.missing.var, function(x) {
-                 c(1:nvar)[-x] 
+                 c(1:nvar)[-x]
              })
              idx.cat.observed.var <- sapply(1:nobs, function(i) {
-                 X[i, idx.observed.var[[i]]]  
+                 X[i, idx.observed.var[[i]]]
              })
              ElnyiGivyjbCasewise <- sapply(1:no.cases.with.missing,function(i) {
                  tmp.id.case <- id.cases.with.missing[i]
@@ -404,57 +425,136 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
              # for the Fmin function
              Fmin <- Fmin - ElnyiGivyjb
 
-        } #end of if (missing =="doubly.robust") 
+        } #end of if (missing =="doubly.robust")
 
+
+    ##### Case 2:
+    #####  mixed ordered + numeric
+    #####  no exogenous covariates
+    #####
+    } else if(nexo == 0L) {
+        # mixed ordered/numeric variables, but no exogenous covariates
+        # - no need to compute 'casewise' (log)likelihoods
+
+        PSTAR <- matrix(0, nvar, nvar)   # utility matrix, to get indices
+        PSTAR[lav_matrix_vech_idx(nvar, diagonal = FALSE)] <- 1:pstar
+        N <- NROW(X)
+
+        logLikPair <- numeric(pstar)  # logl per pair (summed over cases)
+        for(j in seq_len(nvar-1L)) {
+            for(i in (j+1L):nvar) {
+                pstar.idx <- PSTAR[i,j]
+                if(ov.types[i] == "numeric" &&
+                   ov.types[j] == "numeric") {
+                    logLIK <- lav_mvnorm_loglik_data(
+                        Y = X[,c(i,j)], Mu = Mu.hat[c(i,j)],
+                        Sigma = Sigma.hat[c(i,j), c(i,j)], casewise = TRUE)
+                    logLikPair[pstar.idx] <- sum(logLIK, na.rm = TRUE)
+                } else if(ov.types[i] == "numeric" &&
+                          ov.types[j] == "ordered") {
+                    # polyserial correlation
+                    logLIK <- ps_loglik_no_exo(Y1 = X[,i], Y2 = X[,j],
+                                               var.y1 = Sigma.hat[i,i],
+                                               eta.y1 = rep(Mu.hat[i], N),
+                                               th.y2 = TH[ th.idx == j ],
+                                               rho = Cor.hat[i,j])
+                    logLikPair[pstar.idx] <- sum(logLIK, na.rm = TRUE)
+                } else if(ov.types[j] == "numeric" &&
+                          ov.types[i] == "ordered") {
+                    # polyserial correlation
+                    logLIK <- ps_loglik_no_exo(Y1 = X[,j], Y2 = X[,i],
+                                               var.y1 = Sigma.hat[j,j],
+                                               eta.y1 = rep(Mu.hat[j], N),
+                                               th.y2 = TH[ th.idx == i ],
+                                               rho = Cor.hat[i,j])
+                    logLikPair[pstar.idx] <- sum(logLIK, na.rm = TRUE)
+                } else if(ov.types[i] == "ordered" &&
+                          ov.types[j] == "ordered") {
+                    # polychoric correlation
+                    pairwisePI <- pc_PI(rho   = Cor.hat[i,j],
+                                        th.y1 = TH[ th.idx == i ],
+                                        th.y2 = TH[ th.idx == j ])
+                    # avoid zeroes
+                    pairwisePI[ pairwisePI < .Machine$double.eps] <-
+                                             .Machine$double.eps
+                    # note: missing values are just not counted
+                    FREQ <- pc_freq(X[,i], X[,j])
+                    logLikPair[pstar.idx] <- sum(FREQ * log(pairwisePI))
+                }
+            }
+        } # all pairs
+
+        na.idx <- which(is.na(logLikPair))
+        if(length(na.idx) > 0L) {
+            warning("lavaan WARNING: some pairs produces NA values for logl:",
+                    "\n", paste(round(logLikPair, 3), collapse = " "))
+        }
+
+        # sum over pairs
+        logl <- sum(logLikPair)
+
+        # Fmin
+        Fmin <- (-1)*logl
+
+
+    ##### Case 3:
+    #####  mixed ordered + numeric
+    #####  exogenous covariates
+    #####  (conditional.x = TRUE)
     } else {
-        # # order! first i, then j, lav_matrix_vec(table)!
+
+        LIK <- matrix(0, nrow(X), pstar) # likelihood per case, per pair
+        PSTAR <- matrix(0, nvar, nvar)   # utility matrix, to get indices
+        PSTAR[lav_matrix_vech_idx(nvar, diagonal = FALSE)] <- 1:pstar
+        N <- NROW(X)
+
         for(j in seq_len(nvar-1L)) {
             for(i in (j+1L):nvar) {
                 pstar.idx <- PSTAR[i,j]
                 # cat("pstar.idx =", pstar.idx, "i = ", i, " j = ", j, "\n")
-                if(ov.types[i] == "numeric" && 
+                if(ov.types[i] == "numeric" &&
                    ov.types[j] == "numeric") {
                     # ordinary pearson correlation
-                    stop("not done yet")
-                } else if(ov.types[i] == "numeric" && 
+                    LIK[,pstar.idx] <- pp_lik(Y1 = X[,i], Y2 = X[,j],
+                                              var.y1 = Sigma.hat[i,i],
+                                              eta.y1 = rep(Mu.hat[i], N),
+                                              eta.y2 = rep(Mu.hat[j], N),
+                                              var.y2 = Sigma.hat[j,j],
+                                              eXo = eXo,
+                                              rho = Cor.hat[i,j])
+                } else if(ov.types[i] == "numeric" &&
                           ov.types[j] == "ordered") {
                     # polyserial correlation
-                    stop("not done yet")
-                } else if(ov.types[j] == "numeric" && 
+                    ### FIXME: th.y2 should go into ps_lik!!!
+                    LIK[,pstar.idx] <- ps_lik(Y1 = X[,i], Y2 = X[,j],
+                                              var.y1 = Sigma.hat[i,i],
+                                              eta.y1 = rep(Mu.hat[i], N),
+                                              eXo = eXo,
+                                              rho = Cor.hat[i,j])
+                } else if(ov.types[j] == "numeric" &&
                           ov.types[i] == "ordered") {
                     # polyserial correlation
-                    stop("not done yet")
-                } else if(ov.types[i] == "ordered" && 
+                    ### FIXME: th.y1 should go into ps_lik!!!
+                    LIK[,pstar.idx] <- ps_lik(Y1 = X[,j], Y2 = X[,i],
+                                              var.y1 = Sigma.hat[j,j],
+                                              eta.y1 = rep(Mu.hat[j], N),
+                                              eXo = eXo,
+                                              rho = Cor.hat[i,j])
+                } else if(ov.types[i] == "ordered" &&
                           ov.types[j] == "ordered") {
-                    # polychoric correlation
-                    if(nexo == 0L) {
-                        pairwisePI <- pc_PI(rho   = Sigma.hat[i,j], 
-                                            th.y1 = TH[ th.idx == i ],
-                                            th.y2 = TH[ th.idx == j ])
-                        LIK[,pstar.idx] <- pairwisePI[ cbind(X[,i], X[,j]) ]
-                    } else {
-                         LIK[,pstar.idx] <- 
-                             pc_lik_PL_with_cov(Y1          = X[,i],
-                                                Y2          = X[,j],
-                                                Rho         = Sigma.hat[i,j],
-                                                th.y1       = TH[th.idx==i],
-                                                th.y2       = TH[th.idx==j],
-                                                eXo         = eXo,
-                                                PI.y1       = PI[i,],
-                                                PI.y2       = PI[j,],
-                                                missing.ind = missing)
-                             #pc_lik2(Y1 = X[, i], Y2 = X[,j], 
-                             #        rho = Sigma.hat[i, j], 
-                             #        th.y1 = TH[th.idx == i], 
-                             #        th.y2 = TH[th.idx == j], 
-                             #        eXo = eXo, 
-                             #        sl.y1 = PI[i, ], 
-                             #        sl.y2 = PI[j, ])
-                    }
+                     LIK[,pstar.idx] <-
+                         pc_lik_PL_with_cov(Y1          = X[,i],
+                                            Y2          = X[,j],
+                                            Rho         = Sigma.hat[i,j],
+                                            th.y1       = TH[th.idx==i],
+                                            th.y2       = TH[th.idx==j],
+                                            eXo         = eXo,
+                                            PI.y1       = PI[i,],
+                                            PI.y2       = PI[j,],
+                                            missing.ind = missing)
                 }
-                #cat("Done\n")
             }
-        }
+        } # all pairs
 
         # check for zero likelihoods/probabilities
         # FIXME: or should we replace them with a tiny number?
@@ -463,57 +563,56 @@ estimator.PML <- function(Sigma.hat  = NULL,    # model-based var/cov/cor
             attr(OUT, "logl") <- as.numeric(NA)
             return(OUT)
         }
- 
+
         # loglikelihood
         LogLIK.cases <- log(LIK)
- 
+
         # sum over cases
         LogLIK.pairs <- colSums(LogLIK.cases, na.rm = TRUE)
 
         # sum over pairs
-        LogLik <- logl <- sum(LogLIK.pairs)
+        logl <- logl_pairs <- sum(LogLIK.pairs)
 
-        # Fmin
-        Fmin <- (-1)*LogLik
-    }
+        if(missing == "available.cases" && all(ov.types == "ordered") &&
+           nexo != 0L) {
 
-    if(missing == "available.cases" && all(ov.types == "ordered") && 
-       nexo != 0L) {
+           uni_LIK <- matrix(0, nrow(X), ncol(X))
+           for(i in seq_len(nvar)) {
+               uni_LIK[,i] <- uni_lik(Y1    = X[,i],
+                                      th.y1 = TH[th.idx==i],
+                                      eXo   = eXo,
+                                      PI.y1 = PI[i,])
+           }
 
-       uni_LIK <- matrix(0, nrow(X), ncol(X))
-       for(i in seq_len(nvar)) {
-           uni_LIK[,i] <- uni_lik(Y1    = X[,i],
-                                  th.y1 = TH[th.idx==i],
-                                  eXo   = eXo,
-                                  PI.y1 = PI[i,])
-       }
+           if(any(uni_LIK == 0.0, na.rm = TRUE)) {
+               OUT <- +Inf
+               attr(OUT, "logl") <- as.numeric(NA)
+               return(OUT)
+           }
 
-       if(any(uni_LIK == 0.0, na.rm = TRUE)) {
-           OUT <- +Inf
-           attr(OUT, "logl") <- as.numeric(NA)
-           return(OUT)
-       }
+           uni_logLIK_cases <- log(uni_LIK) * lavcache$uniweights.casewise
 
-       uni_logLIK_cases <- log(uni_LIK) * lavcache$uniweights.casewise
+           #sum over cases
+           uni_logLIK_varwise <- colSums(uni_logLIK_cases)
 
-       #sum over cases
-       uni_logLIK_varwise <- colSums(uni_logLIK_cases)
+           #sum over variables
+           uni_logLIK <- sum(uni_logLIK_varwise)
 
-       #sum over variables
-       uni_logLIK <- sum(uni_logLIK_varwise)
-
-       #add with the pairwise part of LogLik
-       LogLik <- logl <- LogLik + uni_logLIK
+           #add with the pairwise part of LogLik
+           logl <- logl_pairs + uni_logLIK
+        }
 
        #we minimise
-       Fmin <- (-1)*LogLik
+       Fmin <- (-1)*logl
     }
 
+
+    # here, we should have two quantities: logl and Fmin
+
     # function value as returned to the minimizer
-    #fx <- -1 * LogLik
     fx <- Fmin
 
-    # attach 'loglikelihood' 
+    # attach 'loglikelihood'
     attr(fx, "logl") <- logl
 
     fx
@@ -537,7 +636,7 @@ estimator.FML <- function(Sigma.hat = NULL,    # model-based var/cov/cor
     cors <- Sigma.hat[lower.tri(Sigma.hat)]
 
     if(any(abs(cors) > 1)) {
-        return(+Inf) 
+        return(+Inf)
     }
 
     nvar <- nrow(Sigma.hat)
@@ -605,9 +704,9 @@ estimator.MML <- function(lavmodel      = NULL,
                           sample.mean.x = NULL,
                           lavcache      = NULL) {
 
-    # compute case-wise likelihoods 
+    # compute case-wise likelihoods
     lik <- lav_model_lik_mml(lavmodel = lavmodel, THETA = THETA, TH = TH,
-               GLIST = GLIST, group = group, lavdata = lavdata, 
+               GLIST = GLIST, group = group, lavdata = lavdata,
                sample.mean = sample.mean, sample.mean.x = sample.mean.x,
                lavcache = lavcache)
 
@@ -620,3 +719,39 @@ estimator.MML <- function(lavmodel      = NULL,
     fx
 }
 
+estimator.2L <- function(lavmodel       = NULL,
+                         GLIST          = NULL,
+                         Lp             = NULL,
+                         lavsamplestats = NULL,
+                         group          = 1L) {
+
+    YLp <- lavsamplestats@YLp[[group]]
+
+    # compute model-implied statistics for all blocks
+    implied <- lav_model_implied(lavmodel, GLIST = GLIST)
+
+    # here, we assume only 2!!! levels, at [[1]] and [[2]]
+    Sigma.W <- implied$cov[[  (group-1)*2 + 1]]
+    Mu.W    <- implied$mean[[ (group-1)*2 + 1]]
+    Sigma.B <- implied$cov[[  (group-1)*2 + 2]]
+    Mu.B    <- implied$mean[[ (group-1)*2 + 2]]
+
+    loglik <- lav_mvnorm_cluster_loglik_samplestats_2l(YLp = YLp, Lp = Lp,
+                  Mu.W = Mu.W, Sigma.W = Sigma.W,
+                  Mu.B = Mu.B, Sigma.B = Sigma.B,
+                  log2pi = FALSE, minus.two = TRUE)
+
+    # minimize
+    objective <- 1 * loglik
+
+    # divide by (N*2)
+    objective <- objective / (lavsamplestats@ntotal * 2)
+
+    # debug only
+    objective <- objective #- 21.10232
+
+    objective
+}
+
+
+
