@@ -18,7 +18,7 @@ lavOptions <- function(x = NULL, default = NULL, mimic = "lavaan") {
             if(length(not.ok) > 0L) {
                 # only warn if multiple options were requested
                 if(length(x) > 1L) {
-                    warning("lavaan WARNING: option `", x[not.ok], 
+                    warning("lavaan WARNING: option `", x[not.ok],
                             "' not available")
                 }
                 x <- x[ -not.ok ]
@@ -70,17 +70,18 @@ lav_options_default <- function(mimic = "lavaan") {
 
                 # summary data
                 sample.cov.rescale = "default",
-                ridge              = 1e-5,
+                ridge              = FALSE,
+                ridge.x            = FALSE,
+                ridge.constant     = "default",
+                ridge.constant.x   = 1e-5,
 
                 # multiple groups
-                group              = NULL,
                 group.label        = NULL,
                 group.equal        = '',
                 group.partial      = '',
                 group.w.free       = FALSE,
 
                 # clusters
-                cluster            = NULL,
                 level.label        = NULL,
 
                 # estimation
@@ -99,6 +100,8 @@ lav_options_default <- function(mimic = "lavaan") {
                 test                   = "default",
                 bootstrap              = 1000L,
                 observed.information   = "hessian",
+                gamma.n.minus.one      = FALSE,
+                #gamma.unbiased         = FALSE,
 
                 # optimization
                 control                = list(),
@@ -107,6 +110,12 @@ lav_options_default <- function(mimic = "lavaan") {
                 optim.force.converged  = FALSE,
                 optim.gradient         = "analytic",
                 optim.init_nelder_mead = FALSE,
+                optim.var.transform    = "none",
+                optim.parscale         = "none",
+                em.iter.max            = 10000L,
+                em.fx.tol              = 1e-08,
+                em.dx.tol              = 1e-04,
+                em.zerovar.offset      = 0.0001,
 
                 # numerical integration
                 integration.ngh        = 21L,
@@ -120,13 +129,23 @@ lav_options_default <- function(mimic = "lavaan") {
                 # zero values
                 zero.add               = "default",
                 zero.keep.margins      = "default",
-                zero.cell.warn         = TRUE,
+                zero.cell.warn         = FALSE, # since 0.6-1
 
                 # starting values
                 start                  = "default",
 
                 # sanity checks
-                check                  = c("start", "post"),
+                check.start            = TRUE,
+                check.post             = TRUE,
+                check.gradient         = TRUE,
+                check.vcov             = TRUE,
+
+                # more models/info
+                h1                     = TRUE,
+                baseline               = TRUE,
+                baseline.conditional.x.free.slopes = TRUE,
+                implied                = TRUE,
+                loglik                 = TRUE,
 
                 # verbosity
                 verbose                = FALSE,
@@ -136,11 +155,11 @@ lav_options_default <- function(mimic = "lavaan") {
     opt
 }
 
-# this function collects and checks the user-provided options/arguments, 
+# this function collects and checks the user-provided options/arguments,
 # and fills in the "default" values, or changes them in an attempt to
 # produce a consistent set of values...
 #
-# returns a list with the named options 
+# returns a list with the named options
 lav_options_set <- function(opt = NULL) {
 
     if(opt$debug) { cat("lavaan DEBUG: lavaanOptions IN\n"); str(opt) }
@@ -156,7 +175,10 @@ lav_options_set <- function(opt = NULL) {
     opt <- lapply(opt, function(x) { if(is.character(x)) tolower(x) else x})
     # except group,group.partial, which may contain capital letters
     opt$group <- opt.old$group
+    opt$group.label <- opt.old$group.label
     opt$group.partial <- opt.old$group.partial
+    opt$cluster <- opt.old$cluster
+
 
     # do.fit implies se="none and test="none" (unless not default)
     if(!opt$do.fit) {
@@ -179,7 +201,7 @@ lav_options_set <- function(opt = NULL) {
         cat("Warning: mimic=\"LISREL\" is not ready yet. Using EQS instead.\n")
         opt$mimic <- "EQS"
     } else {
-        stop("mimic must be \"lavaan\", \"Mplus\" or \"EQS\" \n")
+        stop("lavaan ERROR: mimic must be \"lavaan\", \"Mplus\" or \"EQS\" \n")
     }
 
     # group.equal and group.partial
@@ -201,9 +223,9 @@ lav_options_set <- function(opt = NULL) {
                                          "regressions", "residuals",
                                          "residual.covariances", "thresholds",
                                          "lv.variances", "lv.covariances"))) {
-        # nothing to do 
+        # nothing to do
     } else {
-        stop("unknown value for `group.equal' argument: ",
+        stop("lavaan ERROR: unknown value for `group.equal' argument: ",
              opt$group.equal, "\n")
     }
     if(is.null(opt$group.partial) || nchar(opt$group.partial) == 0L) {
@@ -230,29 +252,148 @@ lav_options_set <- function(opt = NULL) {
         opt$representation <- "LISREL"
     } else if(opt$representation == "lisrel") {
         opt$representation <- "LISREL"
-    } else if(opt$representation == "eqs" || 
+    } else if(opt$representation == "eqs" ||
               opt$representation == "bentler-weeks") {
         opt$representation <- "EQS"
     } else {
-        stop("representation must be \"LISREL\" or \"EQS\" \n")
+        stop("lavaan ERROR: representation must be \"LISREL\" or \"EQS\" \n")
+    }
+
+    # clustered
+    # brute-force override (for now)
+    if(opt$clustered && !opt$multilevel) {
+        opt$meanstructure <- TRUE
+        #opt$missing <- "listwise"
+
+        if(opt$estimator == "mlr") {
+            opt$estimator <- "ml"
+            opt$test <- "yuan.bentler.mplus"
+            opt$se <- "robust.cluster"
+        } else if(opt$estimator == "mlm") {
+            opt$estimator <- "ml"
+            opt$test <- "satorra.bentler"
+            opt$se <- "robust.cluster.sem"
+        }
+
+        # test
+        if(opt$test == "default") {
+            opt$test <- "yuan.bentler.mplus"
+        } else if(opt$test %in% c("none", "standard",
+                                  "satorra.bentler",
+                                  "yuan.bentler","yuan.bentler.mplus")) {
+            # nothing to do
+        } else if(opt$se == "robust") {
+            opt$test <- "yuan.bentler.mplus"
+        } else {
+            stop("lavaan ERROR: `test' argument must one of \"none\", \"yuan.bentler\", \"yuan.bentler.mplus\" or \"satorra.bentler\" in the clustered case")
+        }
+
+        # se
+        if(opt$se == "default") {
+            opt$se <- "robust.cluster"
+        } else if(opt$se %in% c("none", "robust.cluster",
+                                "robust.cluster.sem")) {
+            # nothing to do
+        } else if(opt$se == "robust") {
+            opt$se <- "robust.cluster"
+        } else {
+            stop("lavaan ERROR: `se' argument must one of \"none\", \"robust.cluster\", or \"robust.cluster.sem\" in the clustered case")
+        }
+
+        # information
+        if(opt$information == "default") {
+            if(opt$se == "robust.cluster") {
+                opt$information <- "observed"
+            } else {
+                opt$information <- "expected"
+            }
+        }
+        #} else if(opt$information %in% c("observed", "first.order")) {
+        #    # nothing to do
+        #} else {
+        #    stop("lavaan ERROR: `information' argument must be \"observed\" in the multilevel case (for now)")
+        #}
+
+        #opt$fixed.x = FALSE
+        #opt$control <- list(gradient = "numerical")
+    }
+
+    # multilevel
+    # brute-force override (for now)
+    if(opt$multilevel) {
+        opt$meanstructure <- TRUE
+        opt$missing <- "listwise"
+
+        # test
+        if(opt$test == "default") {
+            opt$test <- "standard"
+        } else if(opt$test %in% c("none", "standard","yuan.bentler")) {
+            # nothing to do
+        } else if(opt$se == "robust") {
+            opt$test <- "yuan.bentler"
+        } else {
+            stop("lavaan ERROR: `test' argument must one of \"none\", \"standard\" or \"yuan.bentler\" in the multilevel case")
+        }
+
+        # se
+        if(opt$se == "default") {
+            opt$se <- "standard"
+        } else if(opt$se %in% c("none", "standard", "robust.huber.white")) {
+            # nothing to do
+        } else if(opt$se == "robust.sem") {
+            opt$se <- "robust.huber.white"
+        } else {
+            stop("lavaan ERROR: `se' argument must one of \"none\", \"standard\" or \"robust.huber.white\" in the multilevel case")
+        }
+
+        # information
+        if(opt$information == "default") {
+            opt$information <- "observed"
+        }
+        #} else if(opt$information %in% c("observed", "first.order")) {
+        #    # nothing to do
+        #} else {
+        #    stop("lavaan ERROR: `information' argument must be \"observed\" in the multilevel case (for now)")
+        #}
+
+        #opt$fixed.x = FALSE
+        #opt$control <- list(gradient = "numerical")
     }
 
 
     # missing
     if(opt$missing == "default") {
         if(opt$mimic == "Mplus" && !opt$categorical &&
-           opt$estimator %in% c("default", "ml", "mlr")) { 
+           opt$estimator %in% c("default", "ml", "mlr")) {
             # since version 5?
-            opt$missing <- "ml" 
+            opt$missing <- "ml"
             # check later if this is ok
         } else {
             opt$missing <- "listwise"
         }
     } else if(opt$missing %in% c("ml", "direct", "fiml")) {
+        #if(opt$categorical && opt$estimator != "mml") {
+        if(opt$categorical) {
+            stop("lavaan ERROR: missing = ", dQuote(opt$missing),
+                 " not available in the categorical setting")
+        }
         opt$missing <- "ml"
         if(opt$estimator %in% c("mlm", "mlmv", "gls", "wls", "wlsm", "wlsmv",
                                 "uls", "ulsm", "ulsmv", "pml")) {
-            stop("lavaan ERROR: missing=\"ml\" is not allowed for estimator MLM, MLMV, GLS, ULS, ULSM, ULSMV, DWLS, WLS, WLSM, WLSMV, PML")
+            stop("lavaan ERROR: missing=\"ml\" is not allowed for estimator ",
+                 dQuote(opt$estimator))
+        }
+    } else if(opt$missing %in% c("ml.x", "direct.x", "fiml.x")) {
+        #if(opt$categorical && opt$estimator != "mml") {
+        if(opt$categorical) {
+            stop("lavaan ERROR: missing = ", dQuote(opt$missing),
+                 " not available in the categorical setting")
+        }
+        opt$missing <- "ml.x"
+        if(opt$estimator %in% c("mlm", "mlmv", "gls", "wls", "wlsm", "wlsmv",
+                                "uls", "ulsm", "ulsmv", "pml")) {
+            stop("lavaan ERROR: missing=\"ml\" is not allowed for estimator ",
+                 dQuote(opt$estimator))
         }
     } else if(opt$missing %in% c("two.stage", "twostage", "two-stage",
                                  "two.step",  "twostep",  "two-step")) {
@@ -264,9 +405,9 @@ lav_options_set <- function(opt = NULL) {
                                 "uls", "ulsm", "ulsmv", "pml", "mml")) {
             stop("lavaan ERROR: missing=\"two.stage\" is not allowed for estimator MLM, MLMV, GLS, ULS, ULSM, ULSMV, DWLS, WLS, WLSM, WLSMV, PML, MML")
         }
-    } else if(opt$missing %in% c("robust.two.stage", "robust.twostage", 
+    } else if(opt$missing %in% c("robust.two.stage", "robust.twostage",
                                  "robust.two-stage", "robust-two-stage",
-                                 "robust.two.step",  "robust.twostep",  
+                                 "robust.two.step",  "robust.twostep",
                                  "robust-two-step")) {
         opt$missing <- "robust.two.stage"
         if(opt$categorical) {
@@ -294,7 +435,7 @@ lav_options_set <- function(opt = NULL) {
     } else if(opt$missing == "available_cases") {
         opt$missing <- "available.cases"
     } else {
-        stop("unknown value for `missing' argument: ", opt$missing, "\n")
+        stop("lavaan ERROR: unknown value for `missing' argument: ", opt$missing, "\n")
     }
 
     # default test statistic
@@ -307,18 +448,22 @@ lav_options_set <- function(opt = NULL) {
         }
     } else if(opt$test %in% c("none", "standard")) {
         # nothing to do
-    } else if(opt$test == "satorra" || 
-              opt$test == "sb" || 
+    } else if(opt$test == "satorra" ||
+              opt$test == "sb" ||
               opt$test == "SB" ||
               opt$test == "satorra.bentler" ||
               opt$test == "satorra-bentler") {
         opt$test <- "satorra.bentler"
-    } else if(opt$test == "yuan" || 
+    } else if(opt$test == "yuan" ||
               opt$test == "yb" ||
               opt$test == "YB" ||
               opt$test == "yuan.bentler" ||
               opt$test == "yuan-bentler") {
         opt$test <- "yuan.bentler"
+    } else if(opt$test == "yuan.bentler.mplus" ||
+              opt$test == "yuan-bentler.mplus" ||
+              opt$test == "yuan-bentler-mplus") {
+        opt$test <- "yuan.bentler.mplus"
     } else if(opt$test == "m.adjusted" ||
               opt$test == "m" ||
               opt$test == "mean.adjusted" ||
@@ -336,27 +481,27 @@ lav_options_set <- function(opt = NULL) {
               opt$test == "scale.shift" ||
               opt$test == "scaled.shifted") {
         opt$test <- "scaled.shifted"
-    } else if(opt$test == "bootstrap" || 
+    } else if(opt$test == "bootstrap" ||
               opt$test == "boot" ||
-              opt$test == "bollen.stine" || 
+              opt$test == "bollen.stine" ||
               opt$test == "bollen-stine") {
         opt$test <- "bollen.stine"
     } else {
-        stop("`test' argument must one of \"none\", \"standard\",
-            \"satorra.bentler\", \"yuan.bentler\",
+        stop("lavaan ERROR: `test' argument must one of \"none\", \"standard\",
+            \"satorra.bentler\", \"yuan.bentler\", \"yuan.bentler.mplus\",
             \"mean.var.adjusted\", \"scaled.shifted\",
             \"bollen.stine\", or \"bootstrap\"")
     }
- 
+
     # check missing
-    if(opt$missing == "ml" && opt$se == "robust.sem") {
+    if(opt$missing %in% c("ml", "ml.x") && opt$se == "robust.sem") {
         warning("lavaan WARNING: missing will be set to ",
                     dQuote("listwise"), " for se = ",
                     dQuote(opt$se) )
         opt$missing <- "listwise"
     }
-    if(opt$missing == "ml" && 
-       opt$test %in% c("satorra.bentler", 
+    if(opt$missing %in% c("ml", "ml.x") &&
+       opt$test %in% c("satorra.bentler",
                        "mean.var.adjusted", "scaled.shifted")) {
         warning("lavaan WARNING: missing will be set to ",
                     dQuote("listwise"), " for test = ",
@@ -375,7 +520,7 @@ lav_options_set <- function(opt = NULL) {
             } else {
                 opt$se <- "robust.two.stage"
             }
-        } else if(opt$missing == "two.stage" && 
+        } else if(opt$missing == "two.stage" &&
                   opt$se      == "two.stage") {
             # nothing to do
         } else if(opt$missing == "robust.two.stage" &&
@@ -418,20 +563,24 @@ lav_options_set <- function(opt = NULL) {
         if(opt$meanstructure == FALSE) {
             # user explicitly wants meanstructure == FALSE
             # check for conflicting arguments
-            if(opt$estimator %in% c("mlm", "mlmv", "mlr", "mlf", "ulsm", "ulsmv", "wlsm", "wlsmv", "pml")) {
-                warning("lavaan WARNING: estimator forces meanstructure = TRUE")
-            }
-            if(opt$missing %in% c("ml", "two.stage")) {
+            #if(opt$estimator %in% c("mlm", "mlmv", "mlr", "mlf", "ulsm", "ulsmv", "wlsm", "wlsmv", "pml")) {
+            #    warning("lavaan WARNING: estimator forces meanstructure = TRUE")
+            #}
+            if(opt$missing %in% c("ml", "ml.x", "two.stage")) {
                 warning("lavaan WARNING: missing argument forces meanstructure = TRUE")
             }
         }
     } else if(opt$meanstructure == "default") {
         # by default: no meanstructure!
-        opt$meanstructure <- FALSE
+        if(opt$estimator == "pml") {
+            opt$meanstructure <- TRUE
+        } else {
+            opt$meanstructure <- FALSE
+        }
         # unless there is a group argument? (added since 0.4-10)
         # if(!is.null(opt$group)) opt$meanstructure <- TRUE
     } else {
-        stop("meanstructure must be TRUE, FALSE or \"default\"\n")
+        stop("lavaan ERROR: meanstructure must be TRUE, FALSE or \"default\"\n")
     }
 
     # estimator and se
@@ -459,12 +608,23 @@ lav_options_set <- function(opt = NULL) {
         opt$estimator <- "ML"
         if(opt$se == "default") {
             opt$se <- "standard"
-        } else if(opt$se %in% c("first.order","bootstrap", "none", 
+        } else if(opt$se %in% c("bootstrap", "none",
                   "external", "standard", "robust.huber.white",
+                  "robust.cluster", "robust.cluster.sem",
                   "two.stage", "robust.two.stage", "robust.sem")) {
             # nothing to do
+        } else if(opt$se == "first.order") {
+            # backwards compatibility
+            opt$se <- "standard"
+            opt$information <- "first.order"
+        } else if(opt$se == "observed") {
+            opt$se <- "standard"
+            opt$information <- "observed"
+        } else if(opt$se == "expected") {
+            opt$se <- "standard"
+            opt$information <- "expected"
         } else if(opt$se == "robust") {
-            if(opt$missing == "ml") {
+            if(opt$missing %in% c("ml", "ml.x")) {
                 opt$se <- "robust.huber.white"
             } else if(opt$missing == "two.stage") {
                 opt$se <- "robust.two.stage"
@@ -472,62 +632,78 @@ lav_options_set <- function(opt = NULL) {
                 opt$se <- "robust.sem"
             }
         } else {
-            stop("unknown value for `se' argument when estimator is ML: ", 
+            stop("lavaan ERROR: unknown value for `se' argument when estimator is ML: ",
                  opt$se, "\n")
         }
 
-    } else if(opt$estimator == "mlm"   || 
-              opt$estimator == "mlmv"  || 
+    } else if(opt$estimator == "mlm"   ||
+              opt$estimator == "mlmv"  ||
               opt$estimator == "mlmvs") {
         est.orig <- opt$estimator
         if(opt$test != "none") {
             if(opt$estimator == "mlm") {
                 opt$test <- "satorra.bentler"
-            } else if(opt$estimator == "mlmv") {          
+            } else if(opt$estimator == "mlmv") {
                 opt$test <- "scaled.shifted"
             } else if(opt$estimator == "mlmvs") {
                 opt$test <- "mean.var.adjusted"
             }
         }
         opt$estimator <- "ML"
-        opt$meanstructure <- TRUE
-        if(opt$se == "bootstrap") stop("use ML estimator for bootstrap")
-        if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(!(opt$information %in% c("expected", "default"))) {
-            warning("lavaan WARNING: information will be set to ",
-                    dQuote("expected"), " for estimator = ", 
-                    dQuote(toupper(est.orig)) )
+        #opt$meanstructure <- TRUE
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use ML estimator for bootstrap")
         }
-        opt$information <- "expected"
+        if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
+        #if(!(opt$information %in% c("expected", "default"))) {
+        #    warning("lavaan WARNING: information will be set to ",
+        #            dQuote("expected"), " for estimator = ",
+        #            dQuote(toupper(est.orig)) )
+        #}
+        #opt$information <- "expected"
+        # in 0.6, we allow for information = "observed" as well
         opt$missing <- "listwise"
     } else if(opt$estimator == "mlf") {
         opt$estimator <- "ML"
-        opt$meanstructure <- TRUE
-        if(opt$se == "bootstrap") stop("use ML estimator for bootstrap")
-        if(opt$se != "none" && opt$se != "external") opt$se <- "first.order"
+        #opt$meanstructure <- TRUE
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use ML estimator for bootstrap")
+        }
+        if(opt$se != "none" && opt$se != "external") {
+            opt$se <- "standard"
+            opt$information <- "first.order"
+        }
     } else if(opt$estimator == "mlr") {
         opt$estimator <- "ML"
-        if(opt$se == "bootstrap") stop("use ML estimator for bootstrap")
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use ML estimator for bootstrap")
+        }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.huber.white"
-        if(opt$test != "none" && opt$se != "external") opt$test <- "yuan.bentler"
-        opt$meanstructure <- TRUE
+        if(opt$test != "none" && opt$se != "external") {
+            if(opt$mimic == "Mplus" || opt$test == "yuan.bentler.mplus") {
+                opt$test <- "yuan.bentler.mplus"
+            } else {
+                opt$test <- "yuan.bentler.mplus" # for now
+            }
+        }
+        #opt$meanstructure <- TRUE
     } else if(opt$estimator == "gls") {
         opt$estimator <- "GLS"
         if(opt$se == "default" || opt$se == "standard") {
             opt$se <- "standard"
-        } else if(opt$se == "none" || 
-                  opt$se == "bootstrap" || 
+        } else if(opt$se == "none" ||
+                  opt$se == "bootstrap" ||
                   opt$se == "external") {
             # nothing to do
         } else {
-            stop("invalid value for `se' argument when estimator is GLS: ", 
+            stop("lavaan ERROR: invalid value for `se' argument when estimator is GLS: ",
                  opt$se, "\n")
         }
         if(!opt$test %in% c("standard","none")) {
-            stop("invalid value for `test' argument when estimator is GLS: ", 
+            stop("lavaan ERROR: invalid value for `test' argument when estimator is GLS: ",
                  opt$test, "\n")
         }
-        opt$missing <- "listwise"       
+        opt$missing <- "listwise"
     } else if(opt$estimator == "ntrls") {
         opt$estimator <- "NTRLS"
         if(opt$se == "default" || opt$se == "standard") {
@@ -537,11 +713,11 @@ lav_options_set <- function(opt = NULL) {
                   opt$se == "external") {
             # nothing to do
         } else {
-            stop("invalid value for `se' argument when estimator is NTRLS: ",
+            stop("lavaan ERROR: invalid value for `se' argument when estimator is NTRLS: ",
                  opt$se, "\n")
         }
         if(!opt$test %in% c("standard","none")) {
-            stop("invalid value for `test' argument when estimator is NTRLS: ",
+            stop("lavaan ERROR: invalid value for `test' argument when estimator is NTRLS: ",
                  opt$test, "\n")
         }
         opt$missing <- "listwise"
@@ -549,8 +725,8 @@ lav_options_set <- function(opt = NULL) {
         opt$estimator <- "WLS"
         if(opt$se == "default" || opt$se == "standard") {
             opt$se <- "standard"
-        } else if(opt$se == "none" || 
-                  opt$se == "bootstrap" || 
+        } else if(opt$se == "none" ||
+                  opt$se == "bootstrap" ||
                   opt$se == "external") {
             # nothing to do
         } else if(opt$se == "robust.sem") {
@@ -558,11 +734,11 @@ lav_options_set <- function(opt = NULL) {
         } else if(opt$se == "robust") {
             opt$se <- "robust.sem"
         } else {
-            stop("invalid value for `se' argument when estimator is WLS: ", 
+            stop("lavaan ERROR: invalid value for `se' argument when estimator is WLS: ",
                  opt$se, "\n")
         }
         if(!opt$test %in% c("standard","none")) {
-            stop("invalid value for `test' argument when estimator is WLS: ", 
+            stop("lavaan ERROR: invalid value for `test' argument when estimator is WLS: ",
                  opt$test, "\n")
         }
         #opt$missing <- "listwise"
@@ -570,8 +746,8 @@ lav_options_set <- function(opt = NULL) {
         opt$estimator <- "DWLS"
         if(opt$se == "default" || opt$se == "standard") {
             opt$se <- "standard"
-        } else if(opt$se == "none" || 
-                  opt$se == "bootstrap" || 
+        } else if(opt$se == "none" ||
+                  opt$se == "bootstrap" ||
                   opt$se == "external") {
             # nothing to do
         } else if(opt$se == "robust.sem") {
@@ -579,40 +755,61 @@ lav_options_set <- function(opt = NULL) {
         } else if(opt$se == "robust") {
             opt$se <- "robust.sem"
         } else {
-            stop("invalid value for `se' argument when estimator is DWLS: ",
+            stop("lavaan ERROR: invalid value for `se' argument when estimator is DWLS: ",
                  opt$se, "\n")
         }
-        if(!opt$test %in% c("standard","none","satorra.bentler", 
+        if(!opt$test %in% c("standard","none","satorra.bentler",
                             "mean.adjusted",
                             "mean.var.adjusted","scaled.shifted")) {
-            stop("invalid value for `test' argument when estimator is DWLS: ",
+            stop("lavaan ERROR: invalid value for `test' argument when estimator is DWLS: ",
                  opt$test, "\n")
         }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "wlsm") {
         opt$estimator <- "DWLS"
-        if(opt$se == "bootstrap") stop("use (D)WLS estimator for bootstrap")
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use (D)WLS estimator for bootstrap")
+        }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test != "none") opt$test <- "satorra.bentler"
+        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted")) {
+            # nothing to do
+        } else if(opt$test != "none") {
+            opt$test <- "satorra.bentler"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "wlsmv") {
         opt$estimator <- "DWLS"
-        if(opt$se == "bootstrap") stop("use (D)WLS estimator for bootstrap")
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use (D)WLS estimator for bootstrap")
+        }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test != "none") opt$test <- "scaled.shifted"
+        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted")) {
+            # nothing to do
+        } else if(opt$test != "none") {
+            opt$test <- "scaled.shifted"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "wlsmvs") {
         opt$estimator <- "DWLS"
-        if(opt$se == "bootstrap") stop("use (D)WLS estimator for bootstrap")
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use (D)WLS estimator for bootstrap")
+        }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test != "none") opt$test <- "mean.var.adjusted"
+        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted")) {
+            # nothing to do
+        } else if(opt$test != "none") {
+            opt$test <- "mean.var.adjusted"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "uls") {
         opt$estimator <- "ULS"
         if(opt$se == "default" || opt$se == "standard") {
             opt$se <- "standard"
-        } else if(opt$se == "none" || 
-                  opt$se == "bootstrap" || 
+        } else if(opt$se == "none" ||
+                  opt$se == "bootstrap" ||
                   opt$se == "external") {
             # nothing to do
         } else if(opt$se == "robust.sem") {
@@ -620,33 +817,54 @@ lav_options_set <- function(opt = NULL) {
         } else if(opt$se == "robust") {
             opt$se <- "robust.sem"
         } else {
-            stop("invalid value for `se' argument when estimator is ULS: ", 
+            stop("lavaan ERROR: invalid value for `se' argument when estimator is ULS: ",
                  opt$se, "\n")
         }
         if(!opt$test %in% c("standard","none", "satorra.bentler",
                             "mean.adjusted",
                             "mean.var.adjusted","scaled.shifted")) {
-            stop("invalid value for `test' argument when estimator is ULS: ",
+            stop("lavaan ERROR: invalid value for `test' argument when estimator is ULS: ",
                  opt$test, "\n")
         }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "ulsm") {
         opt$estimator <- "ULS"
-        if(opt$se == "bootstrap") stop("use ULS estimator for bootstrap")
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use ULS estimator for bootstrap")
+        }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test != "none") opt$test <- "satorra.bentler"
+        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted")) {
+            # nothing to do
+        } else if(opt$test != "none") {
+            opt$test <- "satorra.bentler"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "ulsmv") {
         opt$estimator <- "ULS"
-        if(opt$se == "bootstrap") stop("use ULS estimator for bootstrap")
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use ULS estimator for bootstrap")
+        }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test != "none") opt$test <- "scaled.shifted"
+        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted")) {
+            # nothing to do
+        } else if(opt$test != "none") {
+            opt$test <- "scaled.shifted"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "ulsmvs") {
         opt$estimator <- "ULS"
-        if(opt$se == "bootstrap") stop("use ULS estimator for bootstrap")
+        if(opt$se == "bootstrap") {
+            stop("lavaan ERROR: use ULS estimator for bootstrap")
+        }
         if(opt$se != "none" && opt$se != "external") opt$se <- "robust.sem"
-        if(opt$test != "none") opt$test <- "mean.var.adjusted"
+        if(opt$test %in% c("mean.var.adjusted", "satorra.bentler",
+                           "scaled.shifted")) {
+            # nothing to do
+        } else if(opt$test != "none") {
+            opt$test <- "mean.var.adjusted"
+        }
         #opt$missing <- "listwise"
     } else if(opt$estimator == "pml") {
         opt$estimator <- "PML"
@@ -683,7 +901,7 @@ lav_options_set <- function(opt = NULL) {
         } else if(opt$link %in% c("logit","probit")) {
             # nothing to do
         } else {
-            stop("lavaan ERROR: link must be `logit' or `probit'")    
+            stop("lavaan ERROR: link must be `logit' or `probit'")
         }
         # check for parameterization
         if(opt$parameterization == "default") {
@@ -699,7 +917,7 @@ lav_options_set <- function(opt = NULL) {
             opt$test <- "none"
         }
     } else {
-        stop("unknown value for `estimator' argument: ", opt$estimator, "\n")
+        stop("lavaan ERROR: unknown value for `estimator' argument: ", opt$estimator, "\n")
     }
 
 
@@ -723,7 +941,7 @@ lav_options_set <- function(opt = NULL) {
     # likelihood approach (wishart or normal) + sample.cov.rescale
     if(!opt$estimator %in% c("ML", "REML", "PML", "FML","NTRLS")) {
         if(opt$likelihood != "default") {
-            stop("likelihood argument is only relevant if estimator = ML")
+            stop("lavaan ERROR: likelihood argument is only relevant if estimator = ML")
         }
         if(opt$sample.cov.rescale == "default") {
             opt$sample.cov.rescale <- FALSE
@@ -735,15 +953,15 @@ lav_options_set <- function(opt = NULL) {
             opt$likelihood <- "normal"
         } else if(opt$likelihood == "default") {
            opt$likelihood <- "normal"
-            if(opt$mimic == "EQS"    || 
-               opt$mimic == "LISREL" || 
+            if(opt$mimic == "EQS"    ||
+               opt$mimic == "LISREL" ||
                opt$mimic == "AMOS") {
                 opt$likelihood <- "wishart"
             }
         } else if(opt$likelihood == "wishart" || opt$likelihood == "normal") {
             # nothing to do
         } else {
-            stop("invalid value for `likelihood' argument: ", 
+            stop("lavaan ERROR: invalid value for `likelihood' argument: ",
                  opt$likelihood, "\n")
         }
 
@@ -753,7 +971,7 @@ lav_options_set <- function(opt = NULL) {
                 opt$sample.cov.rescale <- TRUE
             }
         } else if(!is.logical(opt$sample.cov.rescale)) {
-            stop("sample.cov.rescale must be either \"default\", TRUE, or FALSE")
+            stop("lavaan ERROR: sample.cov.rescale must be either \"default\", TRUE, or FALSE")
         } else {
             # nothing to do
         }
@@ -761,18 +979,18 @@ lav_options_set <- function(opt = NULL) {
 
     # information
     if(opt$information == "default") {
-        if(opt$missing == "ml"     || 
-           opt$se == "robust.huber.white"  || 
+        if(opt$missing %in% c("ml", "ml.x") ||
+           opt$se == "robust.huber.white"   ||
            opt$se == "first.order") {
            #nchar(opt$constraints) > 0L) {
             opt$information <- "observed"
         } else {
             opt$information <- "expected"
         }
-    } else if(opt$information %in% c("observed", "expected")) {
+    } else if(opt$information %in% c("observed", "expected", "first.order")) {
         # nothing to do
     } else {
-        stop("information must be either \"expected\" or \"observed\"\n")
+        stop("lavaan ERROR: information must be either \"expected\", \"observed\", or \"first.order\"\n")
     }
 
     if(opt$h1.information == "structured" ||
@@ -788,11 +1006,17 @@ lav_options_set <- function(opt = NULL) {
     #    stop("lavaan ERROR: h1.information.se must be either \"structured\" or \"unstructured\"\n")
     #}
 
-    # check information if se == "robust.sem"
-    if(opt$se == "robust.sem" && opt$information == "observed") {
-        warning("lavaan WARNING: information will be set to ",
-                dQuote("expected"), " for se = ", dQuote(opt$se))
-        opt$information <- "expected"
+    # check information if estimator is uls/wls and friends
+    if(opt$estimator %in% c("ULS", "WLS", "DWLS")) {
+        if(opt$information != "expected") {
+            warning("lavaan WARNING: information will be set to ",
+                    dQuote("expected"), " for estimator = ", dQuote(opt$estimator))
+            opt$information <- "expected"
+        }
+        #if(opt$h1.information != "structured") { # default value
+        #    warning("lavaan WARNING: h1.information is not used if estimator = ", dQuote(opt$estimator))
+        opt$h1.information <- "unstructured"
+        #}
     }
 
     # conditional.x
@@ -809,7 +1033,7 @@ lav_options_set <- function(opt = NULL) {
     } else {
         stop("lavaan ERROR: conditional.x must be TRUE, FALSE or \"default\"\n")
     }
- 
+
     # if conditional.x, always use a meanstructure
     if(opt$conditional.x) {
         opt$meanstructure <- TRUE
@@ -835,21 +1059,21 @@ lav_options_set <- function(opt = NULL) {
 
 
     # meanstructure again
-    if(opt$missing == "ml" || opt$model.type == "growth") {
+    if(opt$missing %in% c("ml", "ml.x") || opt$model.type == "growth") {
         opt$meanstructure <- TRUE
     }
     if("intercepts" %in% opt$group.equal ||
        "means" %in% opt$group.equal) {
         opt$meanstructure <- TRUE
     }
-    if(opt$se == "robust.huber.white" || 
-       opt$se == "robust.sem" ||
-       opt$test == "satorra.bentler" ||
-       opt$test == "mean.var.adjusted" ||
-       opt$test == "scaled.shifted" ||
-       opt$test == "yuan.bentler") {
-        opt$meanstructure <- TRUE
-    }
+    #if(opt$se == "robust.huber.white" ||
+    #   opt$se == "robust.sem" ||
+    #   opt$test == "satorra.bentler" ||
+    #   opt$test == "mean.var.adjusted" ||
+    #   opt$test == "scaled.shifted" ||
+    #   opt$test == "yuan.bentler") {
+    #    opt$meanstructure <- TRUE
+    #}
     stopifnot(is.logical(opt$meanstructure))
     stopifnot(is.logical(opt$verbose))
     stopifnot(is.logical(opt$warn))
@@ -874,7 +1098,7 @@ lav_options_set <- function(opt = NULL) {
        stop("lavaan ERROR: argument `zero.add' must be numeric or \"default\"")
     }
 
-    if(is.character(opt$zero.keep.margins) && 
+    if(is.character(opt$zero.keep.margins) &&
        opt$zero.keep.margins == "default") {
         if(opt$mimic %in% c("lavaan", "Mplus")) {
             opt$zero.keep.margins <- TRUE
