@@ -25,7 +25,7 @@ lav_partable_flat <- function(FLAT = NULL,
     categorical <- FALSE
 
     ### DEFAULT elements: parameters that are typically not specified by
-    ###                   users, but should typically be considered, 
+    ###                   users, but should typically be considered,
     ###                   either free or fixed
 
     # extract `names' of various types of variables:
@@ -33,7 +33,7 @@ lav_partable_flat <- function(FLAT = NULL,
     #lv.names.r   <- lav_partable_vnames(FLAT, type="lv.regular") # regular latent variables
     lv.names.f   <- lav_partable_vnames(FLAT, type="lv.formative") # formative latent variables
     ov.names     <- lav_partable_vnames(FLAT, type="ov")     # observed variables
-    ov.names.x   <- lav_partable_vnames(FLAT, type="ov.x")   # exogenous x covariates 
+    ov.names.x   <- lav_partable_vnames(FLAT, type="ov.x")   # exogenous x covariates
     ov.names.nox <- lav_partable_vnames(FLAT, type="ov.nox")
     lv.names.x   <- lav_partable_vnames(FLAT, type="lv.x")   # exogenous lv
     ov.names.y   <- lav_partable_vnames(FLAT, type="ov.y")   # dependent ov
@@ -51,7 +51,7 @@ lav_partable_flat <- function(FLAT = NULL,
             warning("lavaan WARNING: thresholds are defined for exogenous variables: ", paste(ov.names.ord1[idx], collapse=" "))
         }
     }
- 
+
     if(!is.null(varTable)) {
         ov.names.ord2 <- as.character(varTable$name[ varTable$type == "ordered" ])
         # remove fixed.x variables
@@ -77,7 +77,7 @@ lav_partable_flat <- function(FLAT = NULL,
             nth <- varTable$nlev[ varTable$name == o ] - 1L
             nth.in.partable <- sum(FLAT$op == "|" & FLAT$lhs == o)
             if(nth != nth.in.partable) {
-                stop("lavaan ERROR: expected ", nth, 
+                stop("lavaan ERROR: expected ", max(0,nth),
                      " threshold(s) for variable ",
                      sQuote(o), "; syntax contains ", nth.in.partable, "\n")
             }
@@ -87,6 +87,12 @@ lav_partable_flat <- function(FLAT = NULL,
     if(length(ov.names.ord) > 0L)
         categorical <- TRUE
 
+    # std.lv = TRUE, group.equal includes "loadings": give warning
+    if(ngroups > 1L && std.lv && "loadings" %in% group.equal) {
+        # suggested by Michael Hallquist
+        warning("lavaan WARNING: std.lv = TRUE forces all variances to be unity in all groups, despite group.equal = \"loadings\"")
+    }
+
     lhs <- rhs <- character(0)
 
     # 1. THRESHOLDS (based on varTable)
@@ -109,7 +115,7 @@ lav_partable_flat <- function(FLAT = NULL,
     # 2. default (residual) variances and covariances
 
     # a) (residual) VARIANCES (all ov's except exo, and all lv's)
-    # NOTE: change since 0.5-17: we ALWAYS include the vars in the 
+    # NOTE: change since 0.5-17: we ALWAYS include the vars in the
     #       parameter table; but only if auto.var = TRUE, we set them free
     #if(auto.var) {
         ov.var <- ov.names.nox
@@ -135,12 +141,12 @@ lav_partable_flat <- function(FLAT = NULL,
     }
 
     # d) exogenous x covariates: VARIANCES + COVARIANCES
-    if(!conditional.x && (nx <- length(ov.names.x)) > 0L) {
+    if((nx <- length(ov.names.x)) > 0L) {
         idx <- lower.tri(matrix(0, nx, nx), diag=TRUE)
         lhs <- c(lhs, rep(ov.names.x,  each=nx)[idx]) # fill upper.tri
         rhs <- c(rhs, rep(ov.names.x, times=nx)[idx])
     }
- 
+
     # create 'op' (thresholds come first, then variances)
     op <- rep("~~", length(lhs)); op[seq_len(nth)] <- "|"
 
@@ -148,7 +154,7 @@ lav_partable_flat <- function(FLAT = NULL,
     #    NOTE: - new in 0.5-19: ALWAYS include scaling parameters in partable,
     #            but only free them if auto.delta = TRUE (and parameterization
     #            is "delta"
-    #if(auto.delta && auto.th && length(ov.names.ord) > 0L && 
+    #if(auto.delta && auto.th && length(ov.names.ord) > 0L &&
     #   # length(lv.names) > 0L &&
     #   (ngroups > 1L || any(FLAT$op == "~*~") || parameterization == "theta")) {
     if(length(ov.names.ord) > 0L) {
@@ -159,11 +165,11 @@ lav_partable_flat <- function(FLAT = NULL,
 
     # 3. INTERCEPTS
     if(meanstructure) {
-        if(conditional.x) {
-            ov.int <- ov.names.nox
-        } else {
+        #if(conditional.x) {
+        #    ov.int <- ov.names.nox
+        #} else {
             ov.int <- ov.names
-        }
+        #}
         # auto-remove ordinal variables
         #idx <- which(ov.int %in% ov.names.ord)
         #if(length(idx)) ov.int <- ov.int[-idx]
@@ -178,7 +184,7 @@ lav_partable_flat <- function(FLAT = NULL,
     if(group.w.free) {
         lhs <- c(lhs, "group")
         rhs <- c(rhs, "w")
-         op <- c(op,  "%") 
+         op <- c(op,  "%")
     }
 
     DEFAULT <- data.frame(lhs=lhs, op=op, rhs=rhs,
@@ -194,34 +200,6 @@ lav_partable_flat <- function(FLAT = NULL,
 
     lv.names     <- lav_partable_vnames(FLAT, type="lv")     # latent variables
     ov.names     <- lav_partable_vnames(FLAT, type="ov")     # observed variables
-
-    # check order of covariances: we only fill the upper.tri!
-    cov.idx <- which(op == "~~" & lhs != rhs)
-    for(i in cov.idx) {
-        lv.ov.names <- c(lv.names, ov.names) ### FIXME!!! OK??
-        lv.idx <- match(c(lhs[i], rhs[i]), lv.ov.names)
-        if(lv.idx[1] > lv.idx[2]) { # swap!
-            tmp <- lhs[i]; lhs[i] <- rhs[i]; rhs[i] <- tmp
-        }
-        if(lhs[i] %in% lv.names && rhs[i] %in% lv.names) {
-            lv.idx <- match(c(lhs[i], rhs[i]), lv.names)
-            if(lv.idx[1] > lv.idx[2]) { # swap!
-                tmp <- lhs[i]; lhs[i] <- rhs[i]; rhs[i] <- tmp
-            }
-        } else if(lhs[i] %in% ov.names && rhs[i] %in% ov.names) {
-            ov.idx <- match(c(lhs[i], rhs[i]), ov.names)
-            if(ov.idx[1] > ov.idx[2]) { # swap!
-                tmp <- lhs[i]; lhs[i] <- rhs[i]; rhs[i] <- tmp
-            }
-        } else { # mixed!! # we allow this since 0.4-10
-            lv.ov.names <- c(lv.names, ov.names) ### FIXME!!! OK??
-            lv.idx <- match(c(lhs[i], rhs[i]), lv.ov.names)
-            if(lv.idx[1] > lv.idx[2]) { # swap!
-                tmp <- lhs[i]; lhs[i] <- rhs[i]; rhs[i] <- tmp
-            }
-        }
-    }
-
     USER <- data.frame(lhs=lhs, op=op, rhs=rhs, mod.idx=mod.idx,
                        stringsAsFactors=FALSE)
 
@@ -229,11 +207,11 @@ lav_partable_flat <- function(FLAT = NULL,
     TMP <- USER[,1:3]
     idx <- which(duplicated(TMP))
     if(length(idx) > 0L) {
-        txt <- sapply(1:length(idx), function(i) { 
-            paste("    ", TMP[idx[i],"lhs"], 
-                          TMP[idx[i], "op"], 
+        txt <- sapply(1:length(idx), function(i) {
+            paste("    ", TMP[idx[i],"lhs"],
+                          TMP[idx[i], "op"],
                           TMP[idx[i],"rhs"]) })
-        warning("duplicated elements in model syntax have been ignored:\n", 
+        warning("duplicated elements in model syntax have been ignored:\n",
                 paste(txt, collapse = "\n"))
         USER <- USER[-idx,]
     }
@@ -241,7 +219,7 @@ lav_partable_flat <- function(FLAT = NULL,
     # check for duplicated elements in DEFAULT
     # - FIXME: can we not avoid this somehow??
     # - for example, if the user model includes 'x1 ~~ x1'
-    #   or 'x1 ~ 1' 
+    #   or 'x1 ~ 1'
     # - remove them from DEFAULT
     TMP <- rbind(DEFAULT[,1:3], USER[,1:3])
     idx <- which(duplicated(TMP, fromLast=TRUE)) # idx should be in DEFAULT
@@ -387,9 +365,9 @@ lav_partable_flat <- function(FLAT = NULL,
             ustart[exo.var.idx] <- as.numeric(NA) # should be overriden later!
               free[exo.var.idx] <- 0L
                exo[exo.var.idx] <- 1L
-        } else if(conditional.x) {
-               exo[exo.var.idx] <- 1L
-        }
+        } #else if(conditional.x) {
+          #     exo[exo.var.idx] <- 1L
+        #}
 
         # 2. intercepts
         exo.int.idx  <- which(op == "~1" &
@@ -399,9 +377,9 @@ lav_partable_flat <- function(FLAT = NULL,
             ustart[exo.int.idx] <- as.numeric(NA) # should be overriden later!
               free[exo.int.idx] <- 0L
                exo[exo.int.idx] <- 1L
-        } else if(conditional.x) {
-               exo[exo.int.idx] <- 1L
-        }
+        } #else if(conditional.x) {
+          #     exo[exo.int.idx] <- 1L
+        #}
 
         # 3. regressions ov + lv
         exo.reg.idx <- which(op == "~" &
@@ -416,6 +394,7 @@ lav_partable_flat <- function(FLAT = NULL,
     if(length(ov.names.ord) > 0L) {
         ord.idx <- which(lhs %in% ov.names.ord &
                          op == "~~" &
+                         user == 0L & ## New in 0.6-1
                          lhs == rhs)
         ustart[ord.idx] <- 1L ## FIXME!! or 0?? (0 breaks ex3.12)
           free[ord.idx] <- 0L
@@ -424,7 +403,8 @@ lav_partable_flat <- function(FLAT = NULL,
     # 5c latent response scales of ordinal variables?
     #    by default, all fixed to 1.0
     if(length(ov.names.ord) > 0L) {
-        delta.idx <- which(op == "~*~")
+        delta.idx <- which(op == "~*~" &
+                           user == 0L) ## New in 0.6-1
         ustart[delta.idx] <- 1.0
           free[delta.idx] <- 0L
     }
@@ -523,7 +503,7 @@ lav_partable_flat <- function(FLAT = NULL,
         # for now, only group
         LIST$block <- group
     }
-    
+
     # block columns (typically only group)
     for(block in blocks) {
         if(block == "group") {
